@@ -50,14 +50,13 @@ DO_COMMAND(do_action)
 
 	if (!*left)
 	{
-		tintin_header(ses, " ACTIONS ");
-		show_list(ses, root, PRIORITY);
+		show_list(ses, root, LIST_ACTION);
 	}
 	else if (*left && !*right) 
 	{
-		if (show_node_with_wild(ses, root, left, PRIORITY) == FALSE) 
+		if (show_node_with_wild(ses, left, LIST_ACTION) == FALSE) 
 		{
-			if (show_message(root))
+			if (show_message(ses, LIST_ACTION))
 			{
 				tintin_puts2("#That action is not defined.", ses);
 			}
@@ -65,9 +64,9 @@ DO_COMMAND(do_action)
 	}
 	else
 	{
-		updatenode_list(root, left, right, pr, PRIORITY);
+		updatenode_list(ses, left, right, pr, LIST_ACTION);
 
-		if (show_message(root)) 
+		if (show_message(ses, LIST_ACTION)) 
 		{
 			tintin_printf2(ses, "#Ok. {%s} now triggers {%s} @ {%s}", left, right, pr);
 		}
@@ -92,15 +91,15 @@ DO_COMMAND(do_unaction)
 
 	while ((node = search_node_with_wild(root, left))) 
 	{
-		if (show_message(root))
+		if (show_message(ses, LIST_ACTION))
 		{
 			tintin_printf2(ses, "#Ok. {%s} is no longer a trigger.", node->left);
 		}
-		deletenode_list(root, node);
+		deletenode_list(ses, node, LIST_ACTION);
 		flag = TRUE;
 	}
 
-	if (!flag && show_message(root))
+	if (!flag && show_message(ses, LIST_ACTION))
 	{
 		tintin_printf(ses, "#No match(es) found for {%s}", left);
 	}
@@ -108,7 +107,7 @@ DO_COMMAND(do_unaction)
 }
 
 
-bool check_presub_actions(const char *original, char *line, struct session *ses)
+void check_all_actions(const char *original, char *line, struct session *ses)
 {
 	struct listnode *node;
 	struct listroot *root;
@@ -117,14 +116,9 @@ bool check_presub_actions(const char *original, char *line, struct session *ses)
 
 	for (node = root->update = root->f_node ; node ; node = root->update)
 	{
-		if (atoi(node->pr) >= 5)
-		{
-			return FALSE;
-		}
-
 		root->update = node->next;
 
-		if (check_one_action(line, node->left, ses))
+		if (check_one_action(line, original, node->left, ses))
 		{
 			char buffer[BUFFER_SIZE];
 
@@ -135,14 +129,15 @@ bool check_presub_actions(const char *original, char *line, struct session *ses)
 				tintin_printf2(ses, "[ACTION: %s]", buffer);
 			}
 			parse_input(buffer, ses);
-			return TRUE;
+
+			return;
 		}
 	}
-	return FALSE;
+	return;
 }
 
 
-bool check_postsub_actions(const char *original, char *line, struct session *ses)
+int check_postsub_actions(const char *original, char *line, struct session *ses)
 {
 	struct listnode *node;
 	struct listroot *root;
@@ -153,7 +148,7 @@ bool check_postsub_actions(const char *original, char *line, struct session *ses
 	{
 		root->update = node->next;
 
-		if (check_one_action(line, node->left, ses))
+		if (check_one_action(line, original, node->left, ses))
 		{
 			char buffer[BUFFER_SIZE];
 
