@@ -33,14 +33,6 @@ void init_buffer(struct session *ses, int size)
 {
 	int cnt;
 
-	if (ses == gts)
-	{
-		ses->scroll_max = size;
-		ses->scroll_row = size - 1;
-
-		return;
-	}
-
 	if (ses->scroll_max == size)
 	{
 		return;
@@ -80,18 +72,20 @@ void init_buffer(struct session *ses, int size)
 }
 
 
-void add_line_buffer(struct session *ses, const char *line, bool more_output)
+void add_line_buffer(struct session *ses, const char *line, int more_output)
 {
 	char linebuf[BUFFER_SIZE], linelog[BUFFER_SIZE];
 	char *pti, *pto;
 	int lines;
 
-	if (ses == gts)
+	if (HAS_BIT(ses->flags, SES_FLAG_SCROLLSTOP))
 	{
 		return;
 	}
 
-	if (more_output && strlen(ses->more_output) < BUFFER_SIZE / 2)
+	strcat(ses->more_output, line);
+
+	if (more_output == TRUE && strlen(ses->more_output) < BUFFER_SIZE / 2)
 	{
 		return;
 	}
@@ -103,7 +97,7 @@ void add_line_buffer(struct session *ses, const char *line, bool more_output)
 		reset_hash_table();
 	}
 
-	pti = (char *) line;
+	pti = ses->more_output;
 	pto = linebuf;
 
 	while (*pti != 0)
@@ -135,6 +129,11 @@ void add_line_buffer(struct session *ses, const char *line, bool more_output)
 
 	ses->buffer[ses->scroll_row] = str_hash(linebuf, lines);
 
+	if (more_output == -1)
+	{
+		str_hash_grep(ses->buffer[ses->scroll_row], TRUE);
+	}
+
 	if (ses->logfile)
 	{
 		logit(ses, linelog, ses->logfile);
@@ -193,14 +192,14 @@ DO_COMMAND(do_grep)
 			}
 			grep_min += grep_max * atoi(left);
 			grep_max += grep_max * atoi(left);
-
-			printf("%d %d\n", grep_min, grep_max);
 		}
 		else
 		{
 			arg = get_arg_in_braces(arg, right, TRUE);
 		}
 
+		SET_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
+
 		sprintf(left, "*%s*", right);
 
 		tintin_header(ses, " GREP %s ", right);
@@ -223,6 +222,11 @@ DO_COMMAND(do_grep)
 				break;
 			}
 
+			if (str_hash_grep(ses->buffer[scroll_cnt], FALSE))
+			{
+				continue;
+			}
+
 			if (regexp(left, ses->buffer[scroll_cnt]))
 			{
 				grep_add = str_hash_lines(ses->buffer[scroll_cnt]);
@@ -257,6 +261,11 @@ DO_COMMAND(do_grep)
 				break;
 			}
 
+			if (str_hash_grep(ses->buffer[scroll_cnt], FALSE))
+			{
+				continue;
+			}
+
 			if (regexp(left, ses->buffer[scroll_cnt]))
 			{
 				grep_add = str_hash_lines(ses->buffer[scroll_cnt]);
@@ -274,62 +283,13 @@ DO_COMMAND(do_grep)
 		while (scroll_cnt != ses->scroll_row);
 
 		tintin_header(ses, "");
+
+		DEL_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
 	}
 	return ses;
 }
 
 
-DO_COMMAND(do_buffer)
-{
-	char left[BUFFER_SIZE];
-
-	arg = get_arg_in_braces(arg, left, FALSE);
-
-	switch (left[0])
-	{
-		case 'h':
-			buffer_h();
-			break;
-
-		case 'u':
-			buffer_u();
-			break;
-
-		case 'd':
-			buffer_d();
-			break;
-
-		case 'e':
-			buffer_e();
-			break;
-
-		default:
-			buffer_e();
-			break;
-	}
-
-	return ses;
-
-	if (!*left)
-	{
-		tintin_puts2("#BUFFER, PROVIDE A SCROLL BACK NUMBER.", ses);
-	}
-	else if (!is_number(left))
-	{
-		tintin_printf2(ses, "#BUFFER, '%s' IS NOT A VALID NUMBER.", left);	
-	}
-	else
-	{
-		ses->scroll_line = atoi(left);
-
-		if (show_buffer(ses) == FALSE)
-		{
-			tintin_printf2(ses, "#BUFFER, '%s' IS TOO FAR BACK.", left);
-		}
-	}
-	return ses;
-}
-
 
 int show_buffer(struct session *ses)
 {
@@ -495,6 +455,48 @@ int show_buffer(struct session *ses)
 	return TRUE;
 }
 
+
+DO_COMMAND(do_buffer)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+
+	arg = get_arg_in_braces(arg, left, FALSE);
+	arg = get_arg_in_braces(arg, right, TRUE);
+
+	switch (left[0])
+	{
+		case 'h':
+			buffer_h();
+			break;
+
+		case 'u':
+			buffer_u();
+			break;
+
+		case 'd':
+			buffer_d();
+			break;
+
+		case 'e':
+			buffer_e();
+			break;
+
+		case 'w':
+			do_writebuffer(ses, right);
+			break;
+
+		case 'i':
+			do_hash(ses, right);
+			break;
+
+		default:
+			buffer_e();
+			break;
+	}
+
+	return ses;
+}
+
 void buffer_u(void)
 {
 	int scroll_size, scroll_cnt, buffer_add, buffer_tmp;
