@@ -69,7 +69,7 @@ DO_COMMAND(do_configure)
 			{
 				if (config_table[cnt].config(ses, right, cnt) != NULL)
 				{
-					if (HAS_BIT(ses->list[LIST_CONFIG]->flags, LIST_FLAG_MESSAGE))
+					if (show_message(ses, LIST_CONFIG))
 					{
 						tintin_printf2(ses, "#CONFIG {%s} HAS BEEN SET TO {%s}.", config_table[cnt].name, capitalize(right));
 					}
@@ -97,13 +97,13 @@ DO_CONFIG(config_speedwalk)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG SPEEDWALK <ON|OFF>");
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -123,13 +123,13 @@ DO_CONFIG(config_verbatim)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG VERBATIM <ON|OFF>");
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -148,13 +148,13 @@ DO_CONFIG(config_repeatenter)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {REPEAT ENTER} <ON|OFF>");
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -173,13 +173,13 @@ DO_CONFIG(config_echocommand)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {ECHO COMMAND} <ON|OFF>");
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -198,13 +198,13 @@ DO_CONFIG(config_verbose)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG VERBOSE <ON|OFF>");
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -223,13 +223,13 @@ DO_CONFIG(config_wordwrap)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG WORDWRAP <ON|OFF>");
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -261,7 +261,7 @@ DO_CONFIG(config_log)
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -288,7 +288,7 @@ DO_CONFIG(config_buffersize)
 
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -307,13 +307,13 @@ DO_CONFIG(config_scrolllock)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {SCROLL LOCK} <ON|OFF>");
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -333,7 +333,7 @@ DO_CONFIG(config_connectretry)
 
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -359,7 +359,7 @@ DO_CONFIG(config_packetpatch)
 
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -387,7 +387,7 @@ DO_CONFIG(config_historysize)
 
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, capitalize(arg), buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -400,7 +400,7 @@ DO_CONFIG(config_tintinchar)
 
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, arg, buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, arg, buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -413,7 +413,7 @@ DO_CONFIG(config_verbatimchar)
 
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, arg, buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, arg, buf, LIST_CONFIG);
 
 	return ses;
 }
@@ -426,7 +426,64 @@ DO_CONFIG(config_repeatchar)
 
 	sprintf(buf, "%d", index);
 
-	updatenode_list(ses->list[LIST_CONFIG], config_table[index].name, arg, buf, PRIORITY);
+	updatenode_list(ses, config_table[index].name, arg, buf, LIST_CONFIG);
 
 	return ses;
 }
+
+DO_CONFIG(config_debugtelnet)
+{
+	char buf[NUMBER_SIZE];
+
+	if (!strcasecmp(arg, "ON"))
+	{
+		SET_BIT(ses->flags, SES_FLAG_DEBUGTELNET);
+	}
+	else if (!strcasecmp(arg, "OFF"))
+	{
+		DEL_BIT(ses->flags, SES_FLAG_DEBUGTELNET);
+	}
+	else
+	{
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+
+		return NULL;
+	}
+	sprintf(buf, "%d", index);
+
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
+
+	return ses;
+}
+
+DO_CONFIG(config_convertmeta)
+{
+	char buf[NUMBER_SIZE];
+
+	if (!strcasecmp(arg, "ON"))
+	{
+		rl_initialize();
+		rl_prep_terminal(1);
+
+		SET_BIT(ses->flags, SES_FLAG_CONVERTMETA);
+	}
+	else if (!strcasecmp(arg, "OFF"))
+	{
+		rl_initialize();
+		rl_deprep_terminal();
+
+		DEL_BIT(ses->flags, SES_FLAG_CONVERTMETA);
+	}
+	else
+	{
+		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+
+		return NULL;
+	}
+	sprintf(buf, "%d", index);
+
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
+
+	return ses;
+}
+
