@@ -48,9 +48,9 @@ void push_call(char *f, ...)
 		debug_index = 0;
 	}
 
-	vsprintf(debug_stack[debug_index], f, ap);
+	vsnprintf(debug_stack[debug_index], MAX_DEBUG_SIZE - 1, f, ap);
 
-	debug_stack[debug_index][MAX_STACK_SIZE - 1] = 0;
+/*	debug_stack[debug_index][MAX_DEBUG_SIZE - 1] = 0; */
 
 	debug_index++;
 }
