@@ -59,15 +59,14 @@ DO_COMMAND(do_function)
 
 	if (*left == 0)
 	{
-		tintin_header(ses, " FUNCTIONS ");
-		show_list(ses, root, ALPHA);
+		show_list(ses, root, LIST_FUNCTION);
 	}
 
 	else if (*left && !*right)
 	{
-		if (show_node_with_wild(ses, root, left, ALPHA) == FALSE)
+		if (show_node_with_wild(ses, left, LIST_FUNCTION) == FALSE)
 		{
-			if (show_message(root))
+			if (show_message(ses, LIST_FUNCTION))
 			{
 				tintin_puts2("#THAT FUNCTION IS NOT DEFINED.", ses);
 			}
@@ -88,9 +87,9 @@ DO_COMMAND(do_function)
 		}
 		substitute(ses, right, temp, SUB_FUN);
 
-		updatenode_list(root, left, temp, "0", ALPHA);
+		updatenode_list(ses, left, temp, "0", LIST_FUNCTION);
 
-		if (show_message(root))
+		if (show_message(ses, LIST_FUNCTION))
 		{
 			tintin_printf2(ses, "#OK. {%s} now gives {%s}.",left, temp);
 		}
@@ -112,14 +111,14 @@ DO_COMMAND(do_unfunction)
 
 	while ((node = search_node_with_wild(root, left)) != NULL)
 	{
-		if (show_message(root))
+		if (show_message(ses, LIST_FUNCTION))
 		{
 			tintin_printf2(ses, "#Ok. {%s} is no longer a function.", node->left);
 		}
-		deletenode_list(root, node);
+		deletenode_list(ses, node, LIST_FUNCTION);
 		flag = TRUE;
 	}
-	if (!flag && show_message(root))
+	if (!flag && show_message(ses, LIST_FUNCTION))
 	{
 		tintin_puts2("#THAT FUNCTION IS NOT DEFINED.", ses);
 	}
