@@ -20,7 +20,7 @@
 ******************************************************************************/
 
 /******************************************************************************
-*   file: buffer.c - funtions related to the scroll back buffer               *
+*   file: help.c - funtions related to the internal help file                 *
 *           (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t ++ 2.00              *
 *                  original help files by various persons                     *
 *                   recoded by Igor van den Hoven 2004                        *
@@ -71,8 +71,8 @@ const struct help_type help_table[81] =
 		"         should reguard this action, 0 most important, 9 least important.  If\n"
 		"         no priority is given, a default value of 5 is assigned.\n"
 		"\n"
+		"         If the search string starts with ~, color codes must be matched.\n"
 		"         If the search string starts with ^, the start of the line must match.\n"
-		"\n"
 		"         If the search string ends with ^, the end of the line must match.\n"
 		"\n"
 		"Example: #act {^You are hungry^} {get bread backpack;eat bread}\n"
@@ -89,17 +89,6 @@ const struct help_type help_table[81] =
 		"\n"
 		"Example: #all quit\n"
 	},
-	{
-		"ANTISUBSTITUTE",
-		"Command: #antisubstitute {string}\n"
-		"\n"
-		"         Will cause any line that contains the string specified to not be\n"
-		"         checked for substitutes and gags.  The string search works the same\n"
-		"         way as the #action command.\n"
-		"\n"
-		"Example: #antisub {^You}\n"
-		"         any line starting with 'You' will not be checked for substitutes.\n"
-	},
 	{
 		"BELL",
 		"Command: #bell\n"
@@ -119,6 +108,26 @@ const struct help_type help_table[81] =
 		"Example: #macro {\\e[11~} {#boss}\n"
 		"         Will make you go into boss mode whenever you hit F1\n"
 	},
+	{
+		"BUFFER",
+		"Command: #buffer {home|up|down|end|write filename|info}\n"
+		"         The buffer command allows you to add macros to scroll in case the\n"
+		"         default bindings do not work. The write option allows you to save\n"
+		"         the entire scrollback buffer to file.\n"
+	},
+	{
+		"CLASS",
+		"Command: #class {name} {open|close|read filename|write filename|kill}\n"
+		"\n"
+		"         The {open} option will open a class, all triggers added afterwards\n"
+		"         will be assigned to that class.\n"
+		"         The {close} option will close the given class and make the last\n"
+		"         accessed open class active.\n"
+		"         The {read} option will only read triggers of the given class from file.\n"
+		"         the class when finished.\n"
+		"         The {write} option will write all triggers of the given class to file.\n"
+		"         The {kill} option will delete all triggers of the given class.\n"
+	},
 	{
 		"COLORS",
 		"Format:  <abc>  with a, b, c being parameters\n"
@@ -176,10 +185,10 @@ const struct help_type help_table[81] =
 		"<068>    #<078>                                                                    <068>#\n"
 		"<068>    #<078>                 Original TINTIN code by Peter Unold                <068>#\n"
 		"<068>    #<078>       new code by Bill Reiss, David A. Wagner, Rob Ellsworth,      <068>#\n"
-		"<068>    #<078>           Jeremy C. Jack, Davin Chan, Igor van den Hoven           <068>#\n"
-		"<068>    #<078>                              1994,2004                             <068>#\n"
+		"<068>    #<078>                 Jeremy C. Jack, Igor van den Hoven                 <068>#\n"
+		"<068>    #<078>                              1994,2005                             <068>#\n"
 		"<068>    #<078>                                                                    <068>#\n"
-		"<068>    #<078>                           Version 1.94                             <068>#\n"
+		"<068>    #<078>                           Version 1.95.1                           <068>#\n"
 		"<068>    ######################################################################<088>\n"
 	},
 	{
@@ -307,6 +316,95 @@ const struct help_type help_table[81] =
 		"\n"
 		"         Not every list has debug support yet.\n"
 	},
+	{
+		"FORALL",
+		"Command: #forall {list} {command}\n"
+		"         How this works is best shown with an example\n"
+		"Example: #forall {a b c d} {say $forall}\n"
+		"         This equals: say a;say b;say c;say d;\n"
+		"         Useful for whenever it's useful.\n"
+	},
+	{
+		"FORMAT",
+		"Command: #format {variable} {format} {argument1 argument2 etc}\n"
+		"         Allows you to store a string into a variable in the exact same way\n"
+		"         C's sprintf works with a few enhancements and limitations such as\n"
+		"         no integer operations and a maximum of 20 arguments. If you use format\n"
+		"         inside an alias or action you must escape the %0-9 like: %+4s.\n"
+		"\n"
+		"Example: #format {test} {%+9s} {string}  pad string with up to 9 spaces\n"
+		"         #format {test} {%-9s} {string}  post pad string with up to 9 spaces\n"
+		"         #format {test} {%.8s} {string}  copy at most 8 characters\n"
+		"         #format {test} {%c}   {string}  turn string into a color code\n"
+		"         #format {test} {%d}     {time}  print the current military day\n"
+		"         #format {test} {%h}   {string}  turn text into a header line\n"
+		"         #format {test} {%l}   {string}  lowercase text\n"
+		"         #format {test} {%m}   {string}  perform mathematical calculation\n"
+		"         #format {test} {%n}     {name}  capitalize the first letter\n"
+		"         #format {test} {%r}   {string}  reverse text, hiya = ayih\n"
+		"         #format {test} {%t}     {time}  print the current military time\n"
+		"         #format {test} {%u}   {string}  uppercase text\n"
+		"         #format {time} {%T}             store the epoch time in {time}\n"
+		"         #format {time} {%U}             store the micro epoch time in {time}\n"
+		"         #format {test} {%C}   {colums}  store the screen width in {colums}\n"
+		"         #format {test} {%G}   {number}  perform thousand grouping on {number}\n"
+		"         #format {test} {%L}   {string}  store the string length in {test}\n"
+		"         #format {test} {%R}     {rows}  store the screen height in {rows}\n"
+	},
+	{
+		"FUNCTION",
+		"\n"
+		"Command: #function {name} {operation}\n"
+		"\n"
+		"Functions are a mixture of aliases and variables. The following examples will\n"
+		"have to do to explain how to use functions, result is a special variable you\n"
+		"can use for functions, and if used will place the result at the place where\n"
+		"the function was found.\n"
+		"\n"
+		"Example:\n"
+		"#function {rnd} {#math result &1 - &0;#format result %R $result;#math result $result + &0}\n"
+		"#showme A random number between 100 and 200: @rnd{100 200}\n"
+		"\n"
+		"Functions are treated much like variables. Their value is\n"
+		"a command-line which is executed, and the functions are substituted\n"
+		"by the parameter last sent to the #result command.\n"
+		"\n"
+		"Example:\n"
+		"#function gettime {#format temp %T;#format result %t $temp;#unvar temp}\n"
+		"#showme The time is @gettime{}\n"
+		"This will print the current military time.\n"
+	},
+	{
+		"GETITEMNR",
+		"\n"
+		"Command: getitemnr - retrieves specified element from a list (NEW)\n"
+		"\n"
+		"Syntax:  #getitemnr {destination variable} {item number} {list}\n"
+		"\n"
+		"Example: #geti {dothis} {2} {smile {say Hi!} flip bounce}  <--- sets dothis to 'say Hi!'\n"
+		"\n"
+		"Example: #getl Length {$rndsocial};#math itemnr {1 d $Length};#geti temp {$itemnr}\n"
+		"{$rndsocial};{$temp}\n"
+		"         (will perform a random social from the 'rndsocial' list)\n"
+	},
+	{
+		"GETLISTLENGTH",
+		"\n"
+		"Command: getlistlength - returns the length of a list\n"
+		"\n"
+		"Syntax:  #getlistlength {dest var} {list}\n"
+		"\n"
+		"Example: #getl Length {$alignlist}   <--- returns 3 in the Length variable\n"
+	},
+	{
+		"GREP",
+		"Command: #grep [page] {search string}\n"
+		"         This command allows you to search for matching lines in your scroll\n"
+		"         back buffer. The amount of matches shown equals your screen size. If you\n"
+		"         want to search back further use a page number. You can use wildcards\n"
+		"         for better search results. Default * *'s are placed around the search\n"
+		"         string to make life easier.\n"
+	},
 	{
 		"INFO",
 		"\n"
@@ -322,6 +420,12 @@ const struct help_type help_table[81] =
 		"         Killall deletes all lists.  Useful so you don't have to exit tintin++\n"
 		"         to load up a new command file.\n"
 	},
+	{
+		"LOADPATH",
+		"Command: #loadpath {alias name}\n"
+		"\n"
+		"Load a saved path back into the path, useful for the #walk command.\n"
+	},
 	{
 		"LOG",
 		"\n"
@@ -355,6 +459,19 @@ const struct help_type help_table[81] =
 		"equivalent to\n"
 		"  drop 3.key;drop 2.key;drop 1.key\n"
 	},
+	{
+		"MACRO",
+		"Command: #macro {key sequence} {commands}\n"
+		"         This allows you to make tintin respond to a function key.\n"
+		"\n"
+		"         The key sequence send to the terminal when pressing a function key\n"
+		"         differs for every OS and terminal. To find out what sequence is send\n"
+		"         you can enable the CONVERT META config option.\n"
+		"\n"
+		"Example: Type: #config convert on, type #macro {, press the F1 key and the\n"
+		"         key sequence will be filled in. Finish the macro and type: #config\n"
+		"         convert off, when pressing F1 the entered command should be executed.\n"
+	},
 	{
 		"MAP",
 		"\n"
@@ -458,13 +575,17 @@ const struct help_type help_table[81] =
 		"the walk stack. Notice NW isn't seen as a speedwalk due to capitalization.\n"
 	},
 	{
-		"UNPATH",
+		"PROMPT",
 		"\n"
-		"Command: #unpath\n"
+		"Command: #prompt {text} {new text} {row #}\n"
 		"\n"
-		"Pops the last move off the top of the path\n"
-		"Example\n"
-		"#act {You cannot go that way.} {#unpath}\n"
+		"         Prompt is a feature for split window mode (see #help split), which\n"
+		"         will take the status prompt from the mud, and display it on the status\n"
+		"         line of the split window.  You would define <string> the same way you\n"
+		"         would a substitution (see #help substitution). The row number is\n"
+		"         optional if you use a non standard split mode. In which case a\n"
+		"         positive number draws #row lines above your cursor, and a negative\n"
+		"         number draws #row lines below your topline\n"
 	},
 	{
 		"READ",
@@ -473,6 +594,19 @@ const struct help_type help_table[81] =
 		"Reads a coms file into memory.  The coms file is merged in with the\n"
 		"currently loaded commands.  Duplicate commands are overwritten.\n"
 	},
+	{
+		"REDIT",
+		"Commands: #redit {firstroom|goto|create|map|vmap|find|color|name} {argument}\n"
+		"\n"
+		"#redit firstroom (will create the first room)\n"
+		"#redit goto firstroom (will put you into the room named 'firstroom')\n"
+		"#redit create n (will create a room to the north)\n"
+		"#redit map (will show a little map, 2 rooms big)\n"
+		"#redit vmap (will show a vt map, required #split 5)\n"
+		"#redit find firstroom (searches the shortest path, see #path)\n"
+		"#redit color <code> will set the room's color, see #help colors for info.\n"
+		"#redit link <direction> <room name> will link to an already mapped room\n"
+	},
 	{
 		"REPLACESTRING",
 		"Command: #replacestring {variable} {oldtext} {newtext}\n"
@@ -490,19 +624,47 @@ const struct help_type help_table[81] =
 		"again.\n"
 	},
 	{
-		"LOADPATH",
-		"Command: #loadpath {alias name}\n"
+		"RETURN",
 		"\n"
-		"Load a saved path back into the path, useful for the #walk command.\n"
+		"Command: #return {text}\n"
+		"\n"
+		"The return command can be used in an #if check to break out of a command\n"
+		"string being executed. In a #function you can use #return with an argument\n"
+		"to both break out of the function and set the result variable.\n"
 	},
+	{
+		"SCAN",
+		"\n"
+		"Command: #scan {filename}\n"
+		"\n"
+		"Reads in a filename and sends it to the screen as if it was send by a mud. This\n"
+		"is useful to convert ansi color files to html and viewing log files.\n"
+	},
+	{
+		"SESSION",
+		"\n"
+		"Command: #session {name} {host} {port}\n"
+		"\n"
+		"Starts a telnet session with the given name, host, and port. The name can\n"
+		"be anything you want, except the name of an already existant session.\n"
+		"\n"
+		"Without an argument #session shows the currently active session.\n"
+		"\n"
+		"If you have more than 1 session you can use #session {-|+|number} to switch\n"
+		"between them. Using #{name} allows switching to a session with the given name.\n"
+		"\n"
+		"#{name} {command} allows executing a command with the given session without\n"
+		"changing the active session.\n"
+	},
+
 	{
 		"SHOWME",
 		"\n"
-		"Command: #showme {string}\n"
+		"Command: #showme {string} {row}\n"
 		"\n"
 		"Display the string to the terminal, do not send to the mud.  Useful for\n"
-		"status, warnings, etc.\n"
-		"\n"
+		"status, warnings, etc. The {row} number is optional and works the same way as\n"
+		"the row number of the #prompt trigger.\n"
 		"example:\n"
 		"#action {%0 ultraslays you} {#showme {###### argh! we were ultraslayed ######}}\n"
 	},
@@ -578,7 +740,15 @@ const struct help_type help_table[81] =
 		"replace all occurrences of 'massacres' with 'MASSACRES' and change the colors,\n"
 		"see #help colors, for more information.\n"
 	},
-
+	{
+		"SUSPEND",
+		"\n"
+		"Command: #suspend\n"
+		"\n"
+		"Temporarily suspends tintin++ and returns you to your shell.  The\n"
+		"effect of this command is exactly as if you had typed control-z.\n"
+		"To return to tintin++, type 'fg' at the shell prompt.\n"
+	},
 	{
 		"SYSTEM",
 		"\n"
@@ -593,13 +763,7 @@ const struct help_type help_table[81] =
 		"\n"
 		"Adds a word to the tab completion list, alphabetically sorted.\n"
 	},
-	{
-		"UNTAB",
-		"\n"
-		"format:	       #untab {word}\n"
-		"\n"
-		"Deletes a word from the tab completion list.\n"
-	},
+
 	{
 		"TEXTIN",
 		"\n"
@@ -616,13 +780,6 @@ const struct help_type help_table[81] =
 		"Execute given command every # of seconds.\n"
 		"The epoch time (see help on format) is stored in %0\n"
 	},
-	{
-		"UNTICK",
-		"\n"
-		"Command: #untick {name}\n"
-		"\n"
-		"Removes a ticker from the ticker list.\n"
-	},
 	{
 		"UNACTION",
 		"Command: #unaction {string}\n"
@@ -658,22 +815,11 @@ const struct help_type help_table[81] =
 		"remove all aliases that contain the fragment 'go'\n"
 	},
 	{
-		"UNANTISUBSTITUTE",
+		"UNFUNCTION",
 		"\n"
-		"Command: #unantisubstitute {string}\n"
-		"               #unantisubstitute string\n"
+		"Command: unfunction - remove a function\n"
 		"\n"
-		"Remove antisub(s) from the antisub list which match {string}.  The\n"
-		"'*' character will act as a wildcard and will match any text.\n"
-		"See help wildcard.\n"
-		"\n"
-		"examples:\n"
-		"#unantisub {%0tells you%1}\n"
-		"remove the antisub for '%0tells you%1'\n"
-		"#unantisub *\n"
-		"remove all antisubs\n"
-		"#unantisub {^You*}\n"
-		"remove all antisubs that start with '^You'\n"
+		"Syntax:  #unfunction {func name}\n"
 	},
 	{
 		"UNGAG",
@@ -683,24 +829,6 @@ const struct help_type help_table[81] =
 		"\n"
 		"Exactly the same as #unsubstitute, '#help unsubstitute' for info.\n"
 	},
-	{
-		"UNSUBSTITUTE",
-		"\n"
-		"Command: #unsubstitute {string}\n"
-		"               #unsubstitute string\n"
-		"\n"
-		"Remove substitute(s) from the substitute list which match {string}.  The\n"
-		"'*' character will act as a wildcard and will match any text.\n"
-		"See help wildcard.\n"
-		"\n"
-		"examples:\n"
-		"#unsub {%0tells you%1}\n"
-		"remove the sub for '%0tells you%1'\n"
-		"#unsub *\n"
-		"remove all subs\n"
-		"#unsub {^You*}\n"
-		"remove all subs that start with '^You'\n"
-	},
 	{
 		"UNHIGHLIGHT",
 		"\n"
@@ -719,12 +847,27 @@ const struct help_type help_table[81] =
 		"#unhigh {^You*}\n"
 		"remove all highlights that start with '^You'\n"
 	},
+	{
+		"UNMACRO",
+		"Command: #unmacro {key combination}\n"
+		"         This command removes a macro from the list, but doesn't undo the\n"
+		"         actual macro because readline doesn't allow it.\n"
+	},
 	{
 		"UNPATH",
 		"\n"
 		"Command: #unpath\n"
 		"\n"
-		"Remove the most recent move from the current path.\n"
+		"Pops the last move off the top of the path\n"
+		"Example\n"
+		"#act {You cannot go that way.} {#unpath}\n"
+	},
+	{
+		"UNRPROMPT",
+		"\n"
+		"format: 	#unprompt\n"
+		"\n"
+		"unprompt will remove a prompt from the list.\n"
 	},
 	{
 		"UNSPLIT",
@@ -733,6 +876,38 @@ const struct help_type help_table[81] =
 		"\n"
 		"Exit split mode, and set the screen back to its default terminal settings.\n"
 	},
+	{
+		"UNSUBSTITUTE",
+		"\n"
+		"Command: #unsubstitute {string}\n"
+		"               #unsubstitute string\n"
+		"\n"
+		"Remove substitute(s) from the substitute list which match {string}.  The\n"
+		"'*' character will act as a wildcard and will match any text.\n"
+		"See help wildcard.\n"
+		"\n"
+		"examples:\n"
+		"#unsub {%0tells you%1}\n"
+		"remove the sub for '%0tells you%1'\n"
+		"#unsub *\n"
+		"remove all subs\n"
+		"#unsub {^You*}\n"
+		"remove all subs that start with '^You'\n"
+	},
+	{
+		"UNTAB",
+		"\n"
+		"format:	       #untab {word}\n"
+		"\n"
+		"Deletes a word from the tab completion list.\n"
+	},
+	{
+		"UNTICK",
+		"\n"
+		"Command: #untick {name}\n"
+		"\n"
+		"Removes a ticker from the ticker list.\n"
+	},
 	{
 		"UNVARIABLE",
 		"\n"
@@ -756,106 +931,34 @@ const struct help_type help_table[81] =
 		"\n"
 		"Command: #variable {<variable_name>} {<text to fill variable>}\n"
 		"\n"
-		"Since these are completely new to tintin, and act differently\n"
-		"than anything else, I feel should spend some time on them.\n"
-		"These variables differ from the %0-9 in the fact that you could\n"
-		"specify a full word as a variable name, and they stay in memory for\n"
-		"the full session, unless they are changed, and they can be saved in\n"
-		"the coms file, and can be set to different values if you have 2 or\n"
+		"Variables differ from the %0-9 in the fact that you could specify\n"
+		"a full word as a variable name, and they stay in memory for the\n"
+		"full session unless they are changed. They can be saved in the\n"
+		"coms file, and can be set to different values if you have two or\n"
 		"more sessions running at the same time.  One of the best uses for\n"
 		"variables I think is for spellcasters.\n"
 		"\n"
-		"Currently, you would set up a bunch of aliases like the following.\n"
+		"#alias {target} {#var target %0}\n"
+		"#alias {x}      {flame $target}\n"
+		"#alias {xx}     {flash $target}\n"
+		"#alias {xxx}    {harm  $target}\n"
 		"\n"
-		"#alias {flame} {cast 'flame strike' %0}\n"
-		"#alias {flash} {cast 'call lightning' %0}\n"
-		"#alias {harm} {cast 'harm' %0}\n"
+		"The name of a variable must exist of only letters and numbers in\n"
+		"order to be substituted. If you do not meet these requirements do\n"
+		"not panic, simply encapsulate the variable in braces:\n"
 		"\n"
-		"With the new variables you can do the following:\n"
-		"\n"
-		"#alias {targ} {#var target %0}\n"
-		"#alias {flamet} {flame $target}\n"
-		"#alias {flasht} {flash $target}\n"
-		"#alias {harmt} {harm $target}\n"
-		"\n"
-		"these aliases will be  defined just as they are written, the variables\n"
-		"are not substituted  for until the  alias is found  in your input  and\n"
-		"executed.\n"
-		"\n"
-		"so, if before a battle, you do a:\n"
-		"targ donjonkeeper\n"
-		"then $target is set to donjonkeeper, and any alias you set up with\n"
-		"$target in it will substitute donjonkeeper for every instance of\n"
-		"$target.  Let's say your leader has the following alias set up.\n"
-		"\n"
-		"#alias {setttarg} {#var {target} {%0};gt target=%0}\n"
-		"\n"
-		"if he did a settarg lich, it would set his $target to lich, and\n"
-		"would send a:\n"
-		"\n"
-		"<name> tells your group 'target=lich'\n"
-		"\n"
-		"you could then set an action like so.\n"
-		"\n"
-		"#action {^%0 tells your group 'target=%1'} {targ %1}\n"
-		"\n"
-		"then when your leader executed his alias, it would also set your\n"
-		"variable to the target.\n"
-		"\n"
-		"Another use for variables would be to set a variable $buffer to\n"
-		"whoever the current buffer is.  This would be useful in fights\n"
-		"where the mob switches, or where many rescues are needed.  You\n"
-		"could set up healing aliases with $buffer in them, and set the\n"
-		"$buffer variable in an action that is triggered by each switch,\n"
-		"and each rescue, or just rescues, or whatever.  Then in a\n"
-		"confusing battle, you will have a better shot at healing the\n"
-		"right person.\n"
+		"#variable {cool website} {http://tintin.sourceforge.net}\n"
+		"#chat I was on a ${cool website} yesterday!.\n"
 	},
 	{
-		"WRITE",
-		"\n"
-		"Command: #write {filename}\n"
-		"               #write filename\n"
-		"\n"
-		"Writes all current actions, aliases, subs, antisubs, highlights, and\n"
-		"variables to a coms file, specified by filename.\n"
-	},
-	{
-		"WRITEBUFFER",
-		"\n"
-		"Command: #writebuffer {filename}\n"
-		"\n"
-		"         Writes your scroll back buffer to file. Depending on your\n"
-		"         log data type configuration in #config it will write it in\n"
-		"         either RAW, PLAIN, or HTML format.\n"
-		"\n"
-	},
-	{
-		"WRITESESSION",
-		"\n"
-		"Command: #writesession {filename}\n"
-		"               #writesession filename\n"
-		"\n"
-		"Write all current actions, aliases, subs, antisubs, highlights, and\n"
-		"variables that are specific to your current session to a file.  This\n"
-		"means actions that were not defined when there was no session active.\n"
-	},
-	{
-		"ZAP",
-		"\n"
-		"Command: #zap\n"
-		"\n"
-		"Kill your current session.  If there is no current session, it will cause\n"
-		"the program to terminate.\n"
-	},
-	{
-		"SUSPEND",
-		"\n"
-		"Command: #suspend\n"
-		"\n"
-		"Temporarily suspends tintin++ and returns you to your shell.  The\n"
-		"effect of this command is exactly as if you had typed control-z.\n"
-		"To return to tintin++, type 'fg' at the shell prompt.\n"
+		"WALK",
+		"Command: #walk {forward|backward}\n"
+		"         Walks one step of a current path, forward if no argument is given.\n"
+		"Example: #mark;w;w;w;mark;#savepath backward test;#loadpath test;#3 #walk\n"
+		"         Will make you start mapping, map 3 rooms, save the path backwards,\n"
+		"         And make you walk back the last 3 steps. Everytime you #walk, the\n"
+		"         first element of the loaded path is deleted and executed. You get\n"
+		"         a message when the path is emptied.\n"
 	},
 	{
 		"WILDCARD",
@@ -883,182 +986,32 @@ const struct help_type help_table[81] =
 		"and maybe it'll be included in 2.0.\n"
 	},
 	{
-		"PROMPT",
+		"WRITE",
 		"\n"
-		"Command: #prompt {text} {new text} {row #}\n"
+		"Command: #write {filename}\n"
+		"               #write filename\n"
 		"\n"
-		"         Prompt is a feature for split window mode (see #help split), which\n"
-		"         will take the status prompt from the mud, and display it on the status\n"
-		"         line of the split window.  You would define <string> the same way you\n"
-		"         would a substitution (see #help substitution). The row number is\n"
-		"         optional if you use a non standard split mode. In which case a positive\n"
-		"         number draws #row lines above your cursor, and a negative number draws\n"
-		"         #row lines above your topline\n"
+		"Writes all current actions, aliases, subs, antisubs, highlights, and\n"
+		"variables to a coms file, specified by filename.\n"
 	},
 	{
-		"UNRPROMPT",
+		"WRITESESSION",
 		"\n"
-		"format: 	#unprompt\n"
+		"Command: #writesession {filename}\n"
+		"               #writesession filename\n"
 		"\n"
-		"unprompt will remove a prompt from the list.\n"
+		"Write all current actions, aliases, subs, antisubs, highlights, and\n"
+		"variables that are specific to your current session to a file.  This\n"
+		"means actions that were not defined when there was no session active.\n"
 	},
 	{
-		"FORALL",
-		"Command: #forall {list} {command}\n"
-		"         How this works is best shown with an example\n"
-		"Example: #forall {a b c d} {say $forall}\n"
-		"         This equals: say a;say b;say c;say d;\n"
-		"         Useful for whenever it's useful.\n"
+		"ZAP",
+		"\n"
+		"Command: #zap\n"
+		"\n"
+		"Kill your current session.  If there is no current session, it will cause\n"
+		"the program to terminate.\n"
 	},
-	{
-		"FORMAT",
-		"Command: #format {variable} {format} {argument1 argument2 etc}\n"
-		"         Allows you to store a string into a variable in the exact same way\n"
-		"         C's sprintf works with a few enhancements and limitations such as\n"
-		"         no integer operations and a maximum of 20 arguments. If you use format\n"
-		"         inside an alias or action you must escape the %0-9 like: %+4s.\n"
-		"\n"
-		"Example: #format {test} {%+9s} {string}  pad string with up to 9 spaces\n"
-		"         #format {test} {%-9s} {string}  post pad string with up to 9 spaces\n"
-		"         #format {test} {%.8s} {string}  copy at most 8 characters\n"
-		"         #format {test} {%c}   {string}  turn string into a color code\n"
-		"         #format {test} {%d}     {time}  print the current military day\n"
-		"         #format {test} {%h}   {string}  turn text into a header line\n"
-		"         #format {test} {%l}   {string}  lowercase text\n"
-		"         #format {test} {%m}   {string}  perform mathematical calculation\n"
-		"         #format {test} {%n}     {name}  capitalize the first letter\n"
-		"         #format {test} {%r}   {string}  reverse text, hiya = ayih\n"
-		"         #format {test} {%t}     {time}  print the current military time\n"
-		"         #format {test} {%u}   {string}  uppercase text\n"
-		"         #format {time} {%T}             store the epoch time in {time}\n"
-		"         #format {time} {%U}             store the micro epoch time in {time}\n"
-		"         #format {test} {%C}   {colums}  store the screen width in {colums}\n"
-		"         #format {test} {%G}   {number}  perform thousand grouping on {number}\n"
-		"         #format {test} {%L}   {string}  store the string length in {test}\n"
-		"         #format {test} {%R}     {rows}  store the screen height in {rows}\n"
-
-	},
-	{
-		"GETLISTLENGTH",
-		"\n"
-		"Command: getlistlength - returns the length of a list\n"
-		"\n"
-		"Syntax:  #getlistlength {dest var} {list}\n"
-		"\n"
-		"Example: #getl Length {$alignlist}   <--- returns 3 in the Length variable\n"
-	},
-	{
-		"GETITEMNR",
-		"\n"
-		"Command: getitemnr - retrieves specified element from a list (NEW)\n"
-		"\n"
-		"Syntax:  #getitemnr {destination variable} {item number} {list}\n"
-		"\n"
-		"Example: #geti {dothis} {2} {smile {say Hi!} flip bounce}  <--- sets dothis to 'say Hi!'\n"
-		"\n"
-		"Example: #getl Length {$rndsocial};#math itemnr {1 d $Length};#geti temp {$itemnr}\n"
-		"{$rndsocial};{$temp}\n"
-		"         (will perform a random social from the 'rndsocial' list)\n"
-	},
-	{
-		"FUNCTION",
-		"\n"
-		"Command: #function {name} {operation}\n"
-		"\n"
-		"Functions are a mixture of aliases and variables. The following examples will\n"
-		"have to do to explain how to use functions, result is a special variable you\n"
-		"can use for functions, and if used will place the result at the place where\n"
-		"the function was found.\n"
-		"\n"
-		"Example:\n"
-		"#function {rnd} {#math result &1 - &0;#format result %R $result;#math result $result + &0}\n"
-		"#showme A random number between 100 and 200: @rnd{100 200}\n"
-		"\n"
-		"Functions are treated much like variables. Their value is\n"
-		"a command-line which is executed, and the functions are substituted\n"
-		"by the parameter last sent to the #result command.\n"
-		"\n"
-		"Example:\n"
-		"#function gettime {#format temp %T;#format result %t $temp;#unvar temp}\n"
-		"#showme The time is @gettime{}\n"
-		"This will print the current military time.\n"
-	},
-	{
-		"UNFUNCTION",
-		"\n"
-		"Command: unfunction - remove a function\n"
-		"\n"
-		"Syntax:  #unfunction {func name}\n"
-	},
-	{
-		"SLEEP",
-		"Command: sleep <number>\n"
-		"         This commands will wait the number of seconds before it will process\n"
-		"         any more commands.  This is useful if you want to do something X\n"
-		"         seconds later in time.\n"
-		"Example: charge 'acid blast';#sleep 12.5;fire, for muds with a charge command.\n"
-	},
-
-	{
-		"WALK",
-		"Command: #walk\n"
-		"         Walks one step of a loaded path.\n"
-		"Example: #mark;w;w;w;#savepath backward test;#loadpath test;#3 #walk\n"
-		"         Will make you start mapping, map 3 rooms, save the path backwards,\n"
-		"         And make you walk back the last 3 steps. Everytime you #walk, the\n"
-		"         first element of the loaded path is deleted and executed. You get\n"
-		"         a message when the path is emptied.\n"
-	},
-	{
-		"BUFFER",
-		"Command: #buffer {pages back}\n"
-		"         The buffer command allows you to page through your scrollback buffer\n"
-		"         Per default the home, page-up, page-down, and end keys are turned\n"
-		"         into scroll back macros.\n"
-	},
-	{
-		"GREP",
-		"Command: #grep [page] {search string}\n"
-		"         This command allows you to search for matching lines in your scroll\n"
-		"         back buffer. The amount of matches shown equals your screen size. If you\n"
-		"         want to search back further use a page number. You can use wildcards\n"
-		"         for better search results. Default * *'s are placed around the search\n"
-		"         string to make life easier.\n"
-	},
-	{
-		"MACRO",
-		"Command: #macro {key combination} {commands}\n"
-		"         This allows you to make tintin respond to a key combination.\n"
-		"Example: #macro {\\eOD}  {w\\n}\n"
-		"         left arrow makes you go west, the \\n gives a carriage return.\n"
-		"\n"
-		"         ESC O p would be entered as #macro { \\eOp } {chat hi all\\n}\n"
-		"\n"
-		"         Keypad:      Code:       Keypad:      Code:\n"
-		"           0          ESC O p       F1         ESC [ 1 1 ~\n"
-		"           1          ESC O q       F2         ESC [ 1 2 ~\n"
-		"           2          ESC O r       F3         ESC [ 1 3 ~\n"
-		"           3          ESC O s       F4         ESC [ 1 4 ~\n"
-		"           4          ESC O t       F5         ESC [ 1 5 ~\n"
-		"           5          ESC O u       F6         ESC [ 1 7 ~\n"
-		"           6          ESC O v       F7         ESC [ 1 8 ~\n"
-		"           7          ESC O w       F8         ESC [ 1 9 ~\n"
-		"           8          ESC O x       F9         ESC [ 2 0 ~\n"
-		"           9          ESC O y       F10        ESC [ 2 1 ~\n"
-		"           Up         ESC O A       F11        ESC [ 2 3 ~\n"
-		"          Down        ESC O B       F12        ESC [ 2 4 ~\n"
-		"          Right       ESC O C       -          ESC O m\n"
-		"          Left        ESC O D       ,          ESC O l\n"
-		"                                    .          ESC O n\n"
-	},
-	{
-		"UNMACRO",
-		"Command: #unmacro {key combination}\n"
-		"         This command removes a macro from the list, but doesn't undo the\n"
-		"         actual macro as of yet. Restart tintin in order to do that, I couldn't\n"
-		"         figure out how to do it properly.\n"
-	},
-
 	{
 		"",
 		""
