@@ -54,14 +54,13 @@ DO_COMMAND(do_highlight)
 
 	if (*left == 0)
 	{
-		tintin_header(ses, " HIGHLIGHTS ");
-		show_list(ses, root, PRIORITY);
+		show_list(ses, root, LIST_HIGHLIGHT);
 	}
 	else if (*left && *right == 0)
 	{
-		if (show_node_with_wild(ses, root, left, PRIORITY) == FALSE)
+		if (show_node_with_wild(ses, left, LIST_HIGHLIGHT) == FALSE)
 		{
-			if (show_message(root))
+			if (show_message(ses, LIST_HIGHLIGHT))
 			{
 				tintin_puts2("#THAT HIGHLIGHT IS NOT DEFINED.", ses);
 			}
@@ -76,9 +75,9 @@ DO_COMMAND(do_highlight)
 		}
 		else
 		{
-			updatenode_list(root, left, right, pr, PRIORITY);
+			updatenode_list(ses, left, right, pr, LIST_HIGHLIGHT);
 
-			if (show_message(root))
+			if (show_message(ses, LIST_HIGHLIGHT))
 			{
 				tintin_printf2(ses, "#OK. {%s} IS NOW HIGHLIGHTED {%s} @ {%s}", left, right, pr);
 			}
@@ -101,15 +100,15 @@ DO_COMMAND(do_unhighlight)
 
 	while ((node = search_node_with_wild(root, left)))
 	{
-		if (show_message(root))
+		if (show_message(ses, LIST_HIGHLIGHT))
 		{
 			tintin_printf2(ses, "#OK. {%s} IS NO LONGER A HIGHLIGHT.", node->left);
 		}
-		deletenode_list(root, node);
+		deletenode_list(ses, node, LIST_HIGHLIGHT);
 		flag = TRUE;
 	}
 
-	if (flag == FALSE && show_message(root))
+	if (flag == FALSE && show_message(ses, LIST_HIGHLIGHT))
 	{
 		tintin_puts2("#THAT HIGHLIGHT IS NOT DEFINED.", ses);
 	}
@@ -125,21 +124,21 @@ void check_all_highlights(char *original, char *line, struct session *ses)
 
 	for (node = ses->list[LIST_HIGHLIGHT]->f_node ; node ; node = node->next)
 	{
-		if (check_one_action(original, node->left, ses))
+		if (check_one_action(line, original, node->left, ses))
 		{
-			substitute(ses, node->left, buf1, SUB_VAR|SUB_FUN|SUB_ARG|SUB_ANC);
+			substitute(ses, node->left, buf1, SUB_VAR|SUB_FUN|SUB_ARG|SUB_ANC|SUB_ESC);
 
-			pt1 = original;
-			pt2 = strstr(original, buf1) ? strstr(original, buf1) : buf1;
+			pt1 = strstr(original, buf1) ? original : line;
+			pt2 = strstr(pt1, buf1);
 			*pt2 = 0;
 			pt3 = pt2 + strlen(buf1);
 
 			get_highlight_codes(node->right, buf2);
 
-			strip_non_vt102_codes(original, temp);
+			strip_non_vt102_codes(pt1, temp);
 			strip_vt102_codes_non_graph(temp, buf3);
 
-			sprintf(temp, "%s%s%s\033[0m%s%s", original, buf2, buf1, buf3, pt3);
+			sprintf(temp, "%s%s%s\033[0m%s%s", pt1, buf2, buf1, buf3, pt3);
 
 			strcpy(original, temp);
 
@@ -148,7 +147,7 @@ void check_all_highlights(char *original, char *line, struct session *ses)
 	}
 }
 
-bool get_highlight_codes(const char *string, char *result)
+int get_highlight_codes(const char *string, char *result)
 {
 	int cnt;
 
