@@ -42,6 +42,7 @@ struct listroot *init_list()
 		exit(1);
 	}
 	listhead->flags = LIST_FLAG_DEFAULT;
+
 	return listhead;
 }
 
@@ -49,21 +50,14 @@ struct listroot *init_list()
 	kill list - run throught list and free nodes
 */
 
-void kill_list(struct listroot *listhead)
+void kill_list(struct session *ses, int index)
 {
 	struct listnode *node;
 
-	while ((node = listhead->f_node))
+	while ((node = ses->list[index]->f_node))
 	{
-		UNLINK(node, listhead->f_node, listhead->l_node, next, prev);
-
-		free(node->left);
-		free(node->right);
-		free(node->pr);
-		free(node);
+		deletenode_list(ses, node, index);
 	}
-
-	listhead->count = 0;
 }
 
 /*
@@ -78,7 +72,7 @@ DO_COMMAND(do_killall)
 
 	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
 	{
-		kill_list(ses->list[cnt]);
+		kill_list(ses, cnt);
 
 		if (arg == NULL)
 		{
@@ -98,19 +92,19 @@ DO_COMMAND(do_killall)
 	make a copy of a list - return: ptr to copy
 */
 
-struct listroot *copy_list(struct listroot *sourcelist, int mode)
+struct listroot *copy_list(struct session *ses, struct listroot *sourcelist, int index)
 {
-	struct listroot *resultlist;
-	struct listnode *result;
+	struct listnode *node;
 
-	resultlist = init_list();
+	ses->list[index] = init_list();
 
-	for (result = sourcelist->f_node ; result ; result = result->next)
+	for (node = sourcelist->f_node ; node ; node = node->next)
 	{
-		insertnode_list(resultlist, result->left, result->right, result->pr, mode);
+		insertnode_list(ses, node->left, node->right, node->pr, index);
 	}
-	resultlist->flags = sourcelist->flags;
-	return resultlist;
+	ses->list[index]->flags = sourcelist->flags;
+
+	return ses->list[index];
 }
 
 /*
@@ -120,13 +114,16 @@ struct listroot *copy_list(struct listroot *sourcelist, int mode)
 	- More mods by Scandum
 */
 
-void insertnode_list(struct listroot *listhead, const char *ltext, const char *rtext, const char *prtext, int mode)
+void insertnode_list(struct session *ses, const char *ltext, const char *rtext, const char *prtext, int index)
 {
-	struct listnode *nptr, *newnode;
+	struct listnode *node, *newnode;
+
+	push_call("insertnode_list(%p,%p,%p)",ses,ltext,rtext);
 
 	if ((newnode = calloc(1, sizeof(struct listnode))) == NULL)
 	{
-		fprintf(stderr, "couldn't calloc listhead");
+		fprintf(stderr, "couldn't calloc listhead\n");
+		dump_stack();
 		exit(1);
 	}
 
@@ -134,107 +131,157 @@ void insertnode_list(struct listroot *listhead, const char *ltext, const char *r
 	newnode->right = strdup(rtext);
 	newnode->pr    = strdup(prtext);
 
-	nptr = listhead->f_node;
+	if (ses->class && index != LIST_CLASS)
+	{
+		newnode->class = ses->class;
 
-	listhead->count++;
+		ses->class->data += NODE_FLAG_MAX;
+	}
 
-	switch (mode)
+	ses->list[index]->count++;
+
+	switch (list_table[index].mode)
 	{
 		case PRIORITY:
-			for (nptr = listhead->f_node ; nptr ; nptr = nptr->next)
+			for (node = ses->list[index]->f_node ; node ; node = node->next)
 			{
-				if (atoi(prtext) < atoi(nptr->pr))
+				if (atof(prtext) < atof(node->pr))
 				{
-					INSERT_LEFT(newnode, nptr, listhead->f_node, next, prev);
+					INSERT_LEFT(newnode, node, ses->list[index]->f_node, next, prev);
+					pop_call();
 					return;
 				}
 			}
-			LINK(newnode, listhead->f_node, listhead->l_node, next, prev);
+			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
 			break;
 
 		case ALPHA:
-			for (nptr = listhead->f_node ; nptr ; nptr = nptr->next)
+			for (node = ses->list[index]->f_node ; node ; node = node->next)
 			{
-				if (strcmp(ltext, nptr->left) < 0)
+				if (strcmp(ltext, node->left) < 0)
 				{
-					INSERT_LEFT(newnode, nptr, listhead->f_node, next, prev);
+					INSERT_LEFT(newnode, node, ses->list[index]->f_node, next, prev);
+					pop_call();
 					return;
 				}
 			}
-			LINK(newnode, listhead->f_node, listhead->l_node, next, prev);
+			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
 			break;
 
 		case APPEND:
-			LINK(newnode, listhead->f_node, listhead->l_node, next, prev);
+			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
 			break;
 
 		default:
-			tintin_printf2(NULL, "insertnode_list: bad mode: %d", mode);
+			tintin_printf2(NULL, "insertnode_list: bad list_table mode: %d", list_table[index].mode);
 			break;
 	}
+	pop_call();
+	return;
 }
 
 
-void updatenode_list(struct listroot *listhead, const char *ltext, const char *rtext, const char *prtext, int mode)
+void updatenode_list(struct session *ses, const char *ltext, const char *rtext, const char *prtext, int index)
 {
 	struct listnode *node;
 
-	for (node = listhead->f_node ; node ; node = node->next)
+	push_call("updatenode_list(%p,%p,%p,%p,%p)",ses,ltext,rtext,prtext,index);
+
+	for (node = ses->list[index]->f_node ; node ; node = node->next)
 	{
 		if (strcmp(node->left, ltext) == 0)
 		{
-			if (mode == PRIORITY)
-			{
-				if (strcmp(node->pr, prtext) != 0)
-				{
-					deletenode_list(listhead, node);
-					break;
-				}
-			}
-			else
-			{
-				if (strcmp(node->pr, prtext) != 0)
-				{
-					free(node->pr);
-					node->pr = strdup(prtext);
-				}
-			}
-
 			if (strcmp(node->right, rtext) != 0)
 			{
 				free(node->right);
 				node->right = strdup(rtext);
 			}
+
+			switch (list_table[index].mode)
+			{
+				case PRIORITY:
+					if (atof(node->pr) == atof(prtext))
+					{
+						pop_call();
+						return;
+					}
+					deletenode_list(ses, node, index);
+					insertnode_list(ses, ltext, rtext, prtext, index);
+					break;
+
+				case APPEND:
+					if (strcmp(node->pr, prtext) != 0)
+					{
+						free(node->pr);
+						node->pr = strdup(prtext);
+					}
+					UNLINK(node, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
+					LINK(node, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
+					break;
+
+				case ALPHA:
+					if (strcmp(node->pr, prtext) != 0)
+					{
+						free(node->pr);
+						node->pr = strdup(prtext);
+					}
+					break;
+
+				default:
+					tintin_printf2(ses, "updatenode_list: bad mode");
+					break;
+			}
+			pop_call();
 			return;
 		}
 	}
-	insertnode_list(listhead, ltext, rtext, prtext, mode);
+	insertnode_list(ses, ltext, rtext, prtext, index);
+	pop_call();
+	return;
 }
 
-void deletenode_list(struct listroot *listhead, struct listnode *node)
+void deletenode_list(struct session *ses, struct listnode *node, int index)
 {
-	push_call("deletenode_list(%p,%p)",listhead,node);
+	push_call("deletenode_list(%p,%p,%p)", ses, node, index);
 
-	if ((node->next == NULL && node != listhead->l_node)
-	||  (node->prev == NULL && node != listhead->f_node))
+	if ((node->next == NULL && node != ses->list[index]->l_node)
+	||  (node->prev == NULL && node != ses->list[index]->f_node))
 	{
-		tintin_puts2("#ERROR: delete_nodelist: unlink error.", NULL);		
+		tintin_puts2("#ERROR: delete_nodelist: unlink error.", NULL);
+		dump_stack();
 	}
 	else
 	{
-		if (node == listhead->update)
+		if (node == ses->list[index]->update)
 		{
-			listhead->update = node->next;
+			ses->list[index]->update = node->next;
 		}
 
-		UNLINK(node, listhead->f_node, listhead->l_node, next, prev);
+		if (index == LIST_CLASS)
+		{
+			if (node->data >= NODE_FLAG_MAX)
+			{
+				class_kill(ses, node->left);
+			}
+		}
+
+		if (ses->class == node)
+		{
+			ses->class = NULL;
+		}
+		UNLINK(node, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
 
 		free(node->left);
 		free(node->right);
 		free(node->pr);
 		free(node);
 
-		listhead->count--;
+		if (node->class)
+		{
+			node->class->data -= NODE_FLAG_MAX;
+		}
+
+		ses->list[index]->count--;
 	}
 	pop_call();
 	return;
@@ -288,90 +335,58 @@ struct listnode *searchnode_list_begin(struct listroot *listhead, const char *cp
 	show contens of a node on screen
 */
 
-void shownode_list(struct session *ses, struct listnode *nptr, int mode)
+void shownode_list(struct session *ses, struct listnode *nptr, int index)
 {
-	char buf[BUFFER_SIZE * BUFFER_SAFE], out[BUFFER_SIZE * BUFFER_SAFE], *pti, *pto;
-	int level = 0;
+	char buf[BUFFER_SIZE], out[BUFFER_SIZE];
 
-	switch (mode)
+	switch (list_table[index].args)
 	{
-		case PRIORITY:
-			sprintf(buf, "{%s}<168>={%s} <168>@ {%s}", nptr->left, nptr->right, nptr->pr);
+		case 3:
+			sprintf(buf, "#%s <118>{<088>%s<118>}<168>=<118>{<088>%s<118>} <168>@ <118>{<088>%s<118>}", list_table[index].name, nptr->left, nptr->right, nptr->pr);
+			break;
+		case 2:
+			sprintf(buf, "#%s <118>{<088>%s<118>}<168>=<118>{<088>%s<118>}", list_table[index].name, nptr->left, nptr->right);
+			break;
+		case 1:
+			sprintf(buf, "#%s <118>{<088>%s<118>}", list_table[index].name, nptr->left);
 			break;
 		default:
-			sprintf(buf, "{%s}<168>={%s}", nptr->left, nptr->right);
+			sprintf(buf, "ERROR: list_table[index].args == 0");
 			break;
 	}
 
-	pto = out;
-	pti = buf;
+	substitute(ses, buf, out, SUB_COL);
 
-	while (*pti)
-	{
-		switch (*pti)
-		{
-			case DEFAULT_OPEN:
-				sprintf(pto, "<088><1%d8>%c<088>", level % 5 + 1, DEFAULT_OPEN);
-				level++;
-				pti++;
-				pto += strlen(pto);
-				break;
-
-			case DEFAULT_CLOSE:
-				level--;
-				sprintf(pto, "<088><1%d8>%c<088>", level % 5 + 1, DEFAULT_CLOSE);
-				pti++;
-				pto += strlen(pto);
-				break;
-
-			case '\\':
-				*pto++ = *pti++;
-				if (*pti)
-				{
-					*pto++ = *pti++;
-				}
-				break;
-
-			case ';':
-				sprintf(pto, "<088><168>;<088>");
-				pti++;
-				pto += strlen(pto);
-
-			default:
-				*pto++ = *pti++;
-				break;
-		}
-	}
-	substitute(ses, out, buf, SUB_COL);
-
-	tintin_puts2(buf, ses);
+	tintin_puts2(out, ses);
 }
 
 /*
 	list contens of a list on screen
 */
 
-void show_list(struct session *ses, struct listroot *listhead, int mode)
+void show_list(struct session *ses, struct listroot *listhead, int index)
 {
 	struct listnode *node;
 
+	tintin_header(ses, " %s ", list_table[index].name_multi);
+
 	for (node = listhead->f_node ; node ; node = node->next)
 	{
-		shownode_list(ses, node, mode);
+		shownode_list(ses, node, index);
 	}
 }
 
 
-int show_node_with_wild(struct session *ses, struct listroot *listhead, const char *cptr, int mode)
+int show_node_with_wild(struct session *ses, const char *cptr, int index)
 {
 	struct listnode *node;
 	int flag = FALSE;
 
-	for (node = listhead->f_node ; node ; node = node->next)
+	for (node = ses->list[index]->f_node ; node ; node = node->next)
 	{
 		if (regexp(cptr, node->left))
 		{
-			shownode_list(ses, node, mode);
+			shownode_list(ses, node, index);
 			flag = TRUE;
 		}
 	}
