@@ -40,18 +40,17 @@ void logit(struct session *ses, const char *txt, FILE *file)
 		sprintf(out, "%s\n", txt);
 	}
 	fputs(out, file);
+
+	fflush(file);
 }
 
 
 DO_COMMAND(do_log)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, temp, FALSE);
-	substitute(ses, temp, left, SUB_VAR|SUB_FUN);
-
-	arg = get_arg_in_braces(arg, temp, FALSE);
-	substitute(ses, temp, right, SUB_VAR|SUB_FUN);
+	arg = get_arg_in_braces(arg, left,  FALSE);
+	arg = get_arg_in_braces(arg, right, FALSE);
 
 	if (ses->logfile)
 	{
@@ -106,11 +105,10 @@ DO_COMMAND(do_logline)
 {
 	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, temp, FALSE);
-	substitute(ses, temp, left, SUB_VAR|SUB_FUN);
+	arg = get_arg_in_braces(arg, left, FALSE);
 
 	arg = get_arg_in_braces(arg, temp, TRUE);
-	substitute(ses, temp, right, SUB_VAR|SUB_FUN|SUB_ESC|SUB_COL);
+	substitute(ses, temp, right, SUB_ESC|SUB_COL);
 
 	if (ses->logline)
 	{
@@ -143,11 +141,10 @@ DO_COMMAND(do_logline)
 DO_COMMAND(do_writebuffer)
 {
 	FILE *fp;
-	char left[BUFFER_SIZE], out[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char left[BUFFER_SIZE], out[BUFFER_SIZE];
 	int cnt;
 
-	arg = get_arg_in_braces(arg, temp, FALSE);
-	substitute(ses, temp, left, SUB_VAR|SUB_FUN);
+	arg = get_arg_in_braces(arg, left, FALSE);
 
 	if (ses->buffer == 0)
 	{
@@ -189,6 +186,7 @@ DO_COMMAND(do_writebuffer)
 				if (HAS_BIT(ses->flags, SES_FLAG_LOGPLAIN))
 				{
 					strip_vt102_codes(ses->buffer[cnt], out);
+					strcat(out, "\n");
 				}
 				else if (HAS_BIT(ses->flags, SES_FLAG_LOGHTML))
 				{
@@ -196,7 +194,7 @@ DO_COMMAND(do_writebuffer)
 				}
 				else
 				{
-					strcpy(out, ses->buffer[cnt]);
+					sprintf(out, "%s\n", ses->buffer[cnt]);
 				}
 				fputs(out, fp);
 			}
