@@ -59,11 +59,6 @@ RETSIGTYPE winchhandler(int no_care)
 		{
 			send_sb_naws(ses);
 		}
-
-		if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
-		{
-			init_split(ses, ses->top_row, ses->bot_row);
-		}
 	}
 
 	/*
@@ -82,7 +77,9 @@ RETSIGTYPE winchhandler(int no_care)
 
 RETSIGTYPE tstphandler(int no_care)
 {
-	clean_screen(gtd->ses);
+	printf("\e[r\e[%d;%dH", gtd->ses->rows, 1);
+
+	fflush(stdout);
 
 	kill(0, SIGSTOP);
 
@@ -110,14 +107,12 @@ void trap_handler(int signal)
 	}
 	crashed = TRUE;
 
-/*	reset_scroll_region(gtd->ses); */
+	clean_screen(gtd->ses);
 
 	dump_stack();
 
 	fflush(NULL);
 
-	system("stty echo");
-
 	exit(-1);
 }
 
@@ -162,7 +157,6 @@ int main(int argc, char **argv)
 		syserr("signal SIGINT");
 	}
 
-
 	if (signal(SIGTSTP, tstphandler) == BADSIG)
 	{
 		syserr("signal SIGTSTP");
@@ -192,6 +186,8 @@ int main(int argc, char **argv)
 			sprintf(filestring, "%s/%s", getenv("TINTIN_HISTORY"), HISTORY_FILE);
 		}
 		read_history(filestring);
+
+		add_history("");
 	}
 	srand48(time(NULL));
 
@@ -209,7 +205,7 @@ int main(int argc, char **argv)
 
 	if (argc > arg_num && argv[arg_num])
 	{
-		gtd->ses = do_readnew(gts, argv[arg_num]);
+		gtd->ses = do_read(gts, argv[arg_num]);
 	}
 
 	commandloop();
@@ -252,9 +248,7 @@ void init_tintin(void)
 
 	init_screen_size(gts);
 
-
-	clean_screen(gts);
-	dirty_screen(gts);
+/*	dirty_screen(gts); */
 
 	/*
 		Set application keypad mode and  ESC 0 prefix
@@ -262,7 +256,7 @@ void init_tintin(void)
 
 	printf("\e=\e[?1h");
 
-	readline_echoing_p = HAS_BIT(gts->flags, SES_FLAG_LOCALECHO) ? TRUE : FALSE;
+	SET_BIT(gts->flags, SES_FLAG_VERBOSE);
 
 	do_configure(gts, "{SPEEDWALK}          {ON}");
 	do_configure(gts, "{VERBATIM}          {OFF}");
@@ -270,8 +264,8 @@ void init_tintin(void)
 	do_configure(gts, "{ECHO COMMAND}      {OFF}");
 	do_configure(gts, "{VERBOSE}           {OFF}");
 	do_configure(gts, "{WORDWRAP}           {ON}");
-	do_configure(gts, "{LOG}              {HTML}");
-	do_configure(gts, "{BUFFER SIZE}     {10000}");
+	do_configure(gts, "{LOG}               {RAW}");
+	do_configure(gts, "{BUFFER SIZE}      {5000}");
 	do_configure(gts, "{SCROLL LOCK}        {ON}");
 	do_configure(gts, "{HISTORY SIZE}     {1000}");
 	do_configure(gts, "{CONNECT RETRY}      {60}");
@@ -280,12 +274,14 @@ void init_tintin(void)
 	do_configure(gts, "{VERBATIM CHAR}    {\\\\}");
 	do_configure(gts, "{REPEAT CHAR}         {!}");
 
-	insertnode_list(gts->list[LIST_PATHDIR], "n", "s", "0", ALPHA);
-	insertnode_list(gts->list[LIST_PATHDIR], "e", "w", "0", ALPHA);
-	insertnode_list(gts->list[LIST_PATHDIR], "s", "n", "0", ALPHA);
-	insertnode_list(gts->list[LIST_PATHDIR], "w", "e", "0", ALPHA);
-	insertnode_list(gts->list[LIST_PATHDIR], "u", "d", "0", ALPHA);
-	insertnode_list(gts->list[LIST_PATHDIR], "d", "u", "0", ALPHA);
+	DEL_BIT(gts->flags, SES_FLAG_VERBOSE);
+
+	insertnode_list(gts, "n", "s", "0", LIST_PATHDIR);
+	insertnode_list(gts, "e", "w", "0", LIST_PATHDIR);
+	insertnode_list(gts, "s", "n", "0", LIST_PATHDIR);
+	insertnode_list(gts, "w", "e", "0", LIST_PATHDIR);
+	insertnode_list(gts, "u", "d", "0", LIST_PATHDIR);
+	insertnode_list(gts, "d", "u", "0", LIST_PATHDIR);
 
 	do_help(gtd->ses, "CREDITS");
 }
