@@ -688,7 +688,7 @@ void shortest_path(struct session *ses, char *arg)
 	struct exit_data *exit;
 	int room, vnum;
 
-	kill_list(ses->list[LIST_PATH]);
+	kill_list(ses, LIST_PATH);
 
 	room = find_room(ses, arg);
 
