@@ -61,6 +61,8 @@ DO_COMMAND(do_commands)
 	return ses;
 }
 
+
+
 DO_COMMAND(do_cr)
 {
 	write_line_mud(" ", ses);
@@ -89,14 +91,12 @@ DO_COMMAND(do_suspend)
 
 DO_COMMAND(do_all)
 {
-	char left[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char left[BUFFER_SIZE];
 	struct session *sesptr, *next_ses;
 
 	if (gts->next)
 	{
-		get_arg_in_braces(arg, temp, 1);
-
-		substitute(ses, temp, left, SUB_VAR|SUB_FUN);
+		get_arg_in_braces(arg, left, TRUE);
 
 		for (sesptr = gts->next ; sesptr ; sesptr = next_ses)
 		{
@@ -161,7 +161,7 @@ DO_COMMAND(do_echo)
 {
 	char temp[BUFFER_SIZE];
 
-	sprintf(temp, "%s %s", "result", arg);
+	sprintf(temp, "result %s", arg);
 
 	do_internal_variable(ses, "result <#echo: error>");
 
@@ -176,12 +176,21 @@ DO_COMMAND(do_echo)
 
 DO_COMMAND(do_showme)
 {
-	char result[BUFFER_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], result[BUFFER_SIZE];
 
-	substitute(ses, arg, result, SUB_VAR|SUB_FUN|SUB_COL|SUB_ESC);
+	arg = get_arg_in_braces(arg, left, TRUE);
+	arg = get_arg_in_braces(arg, right, FALSE);
 
-	tintin_puts(result, ses);
+	substitute(ses, left, result, SUB_VAR|SUB_FUN|SUB_COL|SUB_ESC);
 
+	if (*right)
+	{
+		do_one_prompt(ses, result, atoi(right));
+	}
+	else
+	{
+		tintin_puts(result, ses);
+	}
 	return ses;
 }
 
@@ -435,6 +444,20 @@ DO_COMMAND(do_debug)
 	return ses;
 }
 
+DO_COMMAND(do_return)
+{
+	char result[BUFFER_SIZE];
+
+	SET_BIT(ses->flags, SES_FLAG_BREAK);
+
+	if (*arg)
+	{
+		sprintf(result, "%s %s", "result", arg);
+
+		do_internal_variable(ses, result);
+	}
+	return ses;
+}
 
 DO_COMMAND(do_snoop)
 {
@@ -478,11 +501,9 @@ DO_COMMAND(do_snoop)
 
 DO_COMMAND(do_system)
 {
-	char left[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char left[BUFFER_SIZE];
 
-	get_arg_in_braces(arg, temp, TRUE);
-
-	substitute(ses, temp, left, SUB_VAR|SUB_FUN);
+	get_arg_in_braces(arg, left, TRUE);
 
 	if (!*left)
 	{
@@ -490,7 +511,7 @@ DO_COMMAND(do_system)
 		return ses;
 	}
 
-	if (show_message(NULL))
+	if (show_message(ses, -1))
 	{
 		tintin_printf2(ses, "#OK, EXECUTING '%s'", left);
 	}
@@ -508,6 +529,7 @@ DO_COMMAND(do_system)
 	{
 		restore_pos(ses);
 	}
+
 	return ses;
 }
 
