@@ -103,7 +103,7 @@ void write_line_mud(const char *line, struct session *ses)
 
 	if (ses == gts)
 	{
-		tintin_printf2(ses, "#NO SESSION ACTIVE. USE THE %cSESSION COMMAND TO START ONE.", gtd->tintin_char);
+		tintin_printf2(ses, "#NO SESSION ACTIVE. USE: %csession {name} {host port} TO START ONE.", gtd->tintin_char);
 		return;
 	}
 
@@ -113,17 +113,6 @@ void write_line_mud(const char *line, struct session *ses)
 		return;
 	}
 
-	sprintf(outtext, "\033[0m%s", line);
-
-	if (!HAS_BIT(ses->flags, SES_FLAG_LOCALECHO))
-	{
-		memset(&outtext[4], '*', strlen(line));
-	}
-
-	strcat(ses->more_output, outtext);
-
-	add_line_buffer(ses, ses->more_output, FALSE);
-
 	sprintf(outtext, "%s\r\n", line);
 
 	if (write(ses->socket, outtext, strlen(outtext)) == -1)
