@@ -37,11 +37,14 @@
 struct session *parse_input(char *input, struct session *ses)
 {
 	char temp[BUFFER_SIZE], command[BUFFER_SIZE], arg[BUFFER_SIZE];
-	char *input2;
+	char *pti, *pta;
 	struct listnode *ln;
+	int i;
 
 	push_call("[%s] parse_input(%s,%p)",ses->name,input,ses);
 
+	DEL_BIT(ses->flags, SES_FLAG_BREAK);
+
 	if (*input == 0)
 	{
 		write_line_mud("", ses);
@@ -70,20 +73,19 @@ struct session *parse_input(char *input, struct session *ses)
 		return(ses);
 	}
 
-	input2 = input;
+	pti = input;
 
-	while (*input2)
+	while (*pti)
 	{
-		if (*input2 == ';')
+		if (*pti == ';')
 		{
-			input2++;
+			if (*++pti == 0)
+			{
+				break;
+			}
 		}
-		if (*input2 == 0)
-		{
-			break;
-		}
-		input2 = (char *)get_arg_stop_spaces(input2, temp);
-		input2 = (char *)get_arg_all(input2, arg);
+		pti = (char *)get_arg_stop_spaces(pti, temp);
+		pti = (char *)get_arg_all(pti, arg);
 
 		substitute(ses, temp, command, SUB_VAR|SUB_FUN);
 
@@ -93,26 +95,21 @@ struct session *parse_input(char *input, struct session *ses)
 		}
 		else if ((ln = searchnode_list_begin(ses->list[LIST_ALIAS], command, ALPHA)))
 		{
-			int i;
-			char newcommand[BUFFER_SIZE], temp[BUFFER_SIZE];
-			const char *pta = arg;
-
 			strcpy(gtd->vars[0], arg);
 
+			pta = arg;
+
 			for (i = 1 ; i < 10 ; i++)
 			{
-				pta = get_arg_in_braces(pta, temp, FALSE);
-
-				strcpy(gtd->vars[i], temp);
+				pta = (char *) get_arg_in_braces(pta, gtd->vars[i], FALSE);
 			}
-			substitute(ses, ln->right, newcommand, SUB_ARG);
+			substitute(ses, ln->right, command, SUB_ARG);
 
-			if (!strcmp(ln->right, newcommand) && *arg)
+			if (!strcmp(ln->right, command) && *arg)
 			{
-				sprintf(newcommand, "%s %s", ln->right, arg);
+				sprintf(command, "%s %s", ln->right, arg);
 			}
-
-			ses = parse_input(newcommand, ses);
+			ses = parse_input(command, ses);
 		}
 		else if (HAS_BIT(ses->flags, SES_FLAG_SPEEDWALK) && !*arg && is_speedwalk_dirs(command))
 		{
@@ -123,6 +120,10 @@ struct session *parse_input(char *input, struct session *ses)
 			get_arg_with_spaces(arg, arg);
 			write_com_arg_mud(command, arg, ses);
 		}
+		if (HAS_BIT(ses->flags, SES_FLAG_BREAK))
+		{
+			break;
+		}
   	}
 
 	pop_call();
@@ -215,7 +216,7 @@ void process_speedwalk(const char *cp, struct session *ses)
 
 struct session *parse_tintin_command(const char *command, char *arg, struct session *ses)
 {
-	char newcommand[BUFFER_SIZE];
+	char newcommand[BUFFER_SIZE], argument[BUFFER_SIZE];
 	int cmd;
 	struct session *sesptr;
 
@@ -232,11 +233,7 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 			}
 			else
 			{
-				gtd->ses = sesptr;
-
-				tintin_printf(sesptr, "#SESSION '%s' ACTIVATED.", sesptr->name);
-
-				return sesptr;
+				return activate_session(sesptr);
 			}
 		}
 	}
@@ -265,10 +262,19 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 	{
 		if (is_abbrev(command, command_table[cmd].name))
 		{
-			return (ses = (*command_table[cmd].command) (ses, arg));
+			if (HAS_BIT(command_table[cmd].flags, CMD_FLAG_SUB))
+			{
+				substitute(ses, arg, argument, SUB_VAR|SUB_FUN);
+
+				return (ses = (*command_table[cmd].command) (ses, argument));
+			}
+			else
+			{
+				return (ses = (*command_table[cmd].command) (ses, arg));
+			}
 		}
 	}
-	tintin_puts2("#UNKNOWN TINTIN-COMMAND.", ses);
+	tintin_printf2(ses, "#UNKNOWN TINTIN-COMMAND '%s'.", command);
 
 	return ses;
 }
@@ -553,51 +559,33 @@ void write_com_arg_mud(const char *command, const char *argument, struct session
 
 void do_one_line(char *line, struct session *ses)
 {
-	char temp[BUFFER_SIZE];
-	bool antisub = TRUE;
-	bool postact = TRUE;
+	char strip[BUFFER_SIZE];
 
 	push_call("[%s] do_one_line(%p,%p)",ses->name,line,ses);
 
-	strip_vt102_codes(line, temp);
-
-	if (temp[0] == 0)
-	{
-		pop_call();
-		return;
-	}
+	strip_vt102_codes(line, strip);
 
 	if (!HAS_BIT(ses->list[LIST_ACTION]->flags, LIST_FLAG_IGNORE))
 	{
-		postact = !check_presub_actions(line, temp, ses);
+		check_all_actions(line, strip, ses);
 	}
 
-	if (!HAS_BIT(ses->list[LIST_ANTISUBSTITUTE]->flags, LIST_FLAG_IGNORE))
-	{
-		antisub = !check_all_antisubstitutions(line, temp, ses);
-	}
-
-	if (antisub && !HAS_BIT(ses->list[LIST_PROMPT]->flags, LIST_FLAG_IGNORE))
+	if (!HAS_BIT(ses->list[LIST_PROMPT]->flags, LIST_FLAG_IGNORE))
 	{
 		if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
 		{
-			check_all_prompts(line, temp, ses);
+			check_all_prompts(line, strip, ses);
 		}
 	}
 
-	if (antisub && !HAS_BIT(ses->list[LIST_SUBSTITUTE]->flags, LIST_FLAG_IGNORE))
+	if (!HAS_BIT(ses->list[LIST_SUBSTITUTE]->flags, LIST_FLAG_IGNORE))
 	{
-		check_all_substitutions(line, temp, ses);
-	}
-
-	if (postact && !HAS_BIT(ses->list[LIST_ACTION]->flags, LIST_FLAG_IGNORE))
-	{
-		check_postsub_actions(line, temp, ses);
+		check_all_substitutions(line, strip, ses);
 	}
 
 	if (!HAS_BIT(ses->list[LIST_HIGHLIGHT]->flags, LIST_FLAG_IGNORE))
 	{
-		check_all_highlights(line, temp, ses);
+		check_all_highlights(line, strip, ses);
 	}
 	pop_call();
 	return;
