@@ -46,7 +46,7 @@ DO_COMMAND(do_mark)
 	}
 	else
 	{
-		kill_list(ses->list[LIST_PATH]);
+		kill_list(ses, LIST_PATH);
 
 		tintin_puts2("#YOU ARE NOW MAPPING.", ses);
 	}
@@ -174,14 +174,14 @@ DO_COMMAND(do_loadpath)
 
 	if ((node = searchnode_list(ses->list[LIST_ALIAS], left)) == NULL)
 	{
-		if (show_message(root))
+		if (show_message(ses, LIST_PATH))
 		{
 			tintin_printf2(ses, "#ALIAS {%s} NOT FOUND.", left);
 		}
 	}
 	else
 	{
-		kill_list(root);
+		kill_list(ses, LIST_PATH);
 
 		arg = node->right;
 
@@ -202,7 +202,7 @@ DO_COMMAND(do_loadpath)
 			}
 		}
 
-		if (HAS_BIT(ses->list[LIST_PATH]->flags, LIST_FLAG_MESSAGE))
+		if (show_message(ses, LIST_PATH))
 		{
 			tintin_printf2(ses, "#PATH WITH %d NODES LOADED.", root->count);
 		}
@@ -253,7 +253,7 @@ DO_COMMAND(do_unpath)
 	if (root->l_node)
 	{
 		tintin_printf2(ses, "#OK.  DELETED MOVE {%s}", root->l_node->left);
-		deletenode_list(root, root->l_node);
+		deletenode_list(ses, root->l_node, LIST_PATH);
 	}
 	else
 	{
@@ -267,22 +267,42 @@ DO_COMMAND(do_walk)
 {
 	char left[BUFFER_SIZE];
 	struct listroot *root;
+	long long flags;
 
 	root = ses->list[LIST_PATH];
 
+	arg = get_arg_in_braces(arg, left, FALSE);
+
 	if (root->f_node == NULL)
 	{
 		tintin_puts("#END OF PATH.", ses);
 	}
 	else
 	{
+		flags = ses->flags;
+
 		DEL_BIT(ses->flags, SES_FLAG_MAPPING);
 
-		strcpy(left, root->f_node->left);
+		switch (tolower(*left))
+		{
+			case 'b':
+				strcpy(left, root->l_node->right);
+				parse_input(left, ses);
+				deletenode_list(ses, root->l_node, LIST_PATH);
+				break;
 
-		parse_input(left, ses);
+			case '\0':
+			case 'f':
+				strcpy(left, root->f_node->left);
+				parse_input(left, ses);
+				deletenode_list(ses, root->f_node, LIST_PATH);
+				break;
 
-		deletenode_list(root, root->f_node);
+			default:
+				tintin_printf2(ses, "#SYNTAX: #WALK {FORWARD|BACKWARD}");
+				break;
+		}
+		ses->flags = flags;
 	}
 	return ses;
 }
@@ -300,14 +320,13 @@ DO_COMMAND(do_pathdir)
 
 	if (!*left)
 	{
-		tintin_puts2("#Defined pathdirs:", ses);
-		show_list(ses, root, ALPHA);
+		show_list(ses, root, LIST_PATHDIR);
 	}
 	else if (*left && !*right)
 	{
-		if (show_node_with_wild(ses, root, left, ALPHA) == FALSE)
+		if (show_node_with_wild(ses, left, LIST_PATHDIR) == FALSE)
 		{
-			if (show_message(root))
+			if (show_message(ses, LIST_PATHDIR))
 			{
 				tintin_printf2(ses, "#NO MATCH(ES) FOUND FOR {%s}", left);
 			}
@@ -315,9 +334,9 @@ DO_COMMAND(do_pathdir)
 	}
 	else
 	{
-		updatenode_list(root, left, right, "0", ALPHA);
+		updatenode_list(ses, left, right, "0", LIST_PATHDIR);
 
-		if (show_message(root))
+		if (show_message(ses, LIST_PATHDIR))
 		{
 			tintin_printf2(ses, "#OK. DIRECTION {%s} WILL BE REVERSED AS {%s}", left, right);
 		}
