@@ -36,14 +36,12 @@
 
 DO_COMMAND(do_session)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
 	struct session *sesptr;
+	int cnt;
 
-	arg = get_arg_in_braces(arg, temp,  FALSE);
-	substitute(ses, temp, left, SUB_VAR|SUB_FUN);
-
-	arg = get_arg_in_braces(arg, temp, TRUE);
-	substitute(ses, temp, right, SUB_VAR|SUB_FUN);
+	arg = get_arg_in_braces(arg, left,  FALSE);
+	arg = get_arg_in_braces(arg, right, TRUE);
 
 	if (!*left)
 	{
@@ -56,18 +54,28 @@ DO_COMMAND(do_session)
 	}
 	else if (*left && *right == 0)
 	{
-		for (sesptr = gts->next ; sesptr; sesptr = sesptr->next)
+		if (*left == '+')
 		{
-			if (!strcmp(sesptr->name, left))
+			return activate_session(ses->next ? ses->next : gts->next);
+		}
+
+		if (*left == '-')
+		{
+			return activate_session(ses->prev ? ses->prev : gts->prev);
+		}
+
+		if (is_number(left))
+		{
+			for (cnt = 0, sesptr = gts ; sesptr ; cnt++, sesptr = sesptr->next)
 			{
-				show_session(ses, sesptr);
-				break;
+				if (cnt == atoi(left))
+				{
+					return activate_session(sesptr);
+				}
 			}
 		}
-		if (!sesptr)
-		{
-			tintin_puts("#THAT SESSION IS NOT DEFINED.", ses);
-		}
+
+		tintin_puts("#THAT SESSION IS NOT DEFINED.", ses);
 	}
 	else
 	{
@@ -130,20 +138,29 @@ struct session *newactive_session(void)
 {
 	if (gts->next)
 	{
-		gtd->ses = gts->next;
-		tintin_printf(gtd->ses, "#SESSION '%s' ACTIVATED.", gts->next->name);
+		activate_session(gts->next);
 	}
 	else
 	{
 		gtd->ses = gts;
+		dirty_screen(gtd->ses);
 		tintin_puts("#THERE'S NO ACTIVE SESSION NOW.", NULL);
 	}
-	clean_screen(gtd->ses);
-	dirty_screen(gtd->ses);
 
 	return gtd->ses;
 }
 
+struct session *activate_session(struct session *ses)
+{
+	gtd->ses = ses;
+
+	dirty_screen(ses);
+
+	tintin_printf(ses, "#SESSION '%s' ACTIVATED.", ses->name);
+
+	return ses;
+}
+
 /**********************/
 /* open a new session */
 /**********************/
@@ -184,14 +201,19 @@ struct session *new_session(const char *name, const char *address, struct sessio
 
 	for (cnt = 0 ; cnt < LIST_ALL ; cnt++)
 	{
-		newsession->list[cnt] = copy_list(gts->list[cnt], list_table[cnt].mode);
+		copy_list(newsession, gts->list[cnt], cnt);
 	}
 
 	init_screen_size(newsession);
-	clean_screen(newsession);
-	dirty_screen(newsession);
+
+	if (HAS_BIT(gts->flags, SES_FLAG_SPLIT))
+	{
+		init_split(newsession, gts->top_row, gts->bot_row);
+	}
 	init_buffer(newsession, gts->scroll_max);
 
+	dirty_screen(newsession);
+
 	connect_session(newsession);
 
 	return gtd->ses;
@@ -277,10 +299,6 @@ void cleanup_session(struct session *ses)
 
 	UNLINK(ses, gts->next, gts->prev, next, prev);
 
-	tintin_printf(ses, "\n\r#SESSION '%s' DIED.", ses->name);
-
-	do_killall(ses, NULL);
-
 	if (ses->socket)
 	{
 		if (close(ses->socket) == -1)
@@ -289,6 +307,15 @@ void cleanup_session(struct session *ses)
 		}
 	}
 
+	tintin_printf(ses, "\n\r#SESSION '%s' DIED.", ses->name);
+
+	if (ses == gtd->ses)
+	{
+		gtd->ses = newactive_session();
+	}
+
+	do_killall(ses, NULL);
+
 	if (ses->logfile)
 	{
 		fclose(ses->logfile);
@@ -299,20 +326,15 @@ void cleanup_session(struct session *ses)
 		fclose(ses->logline);
 	}
 
+
 	free(ses->name);
 	free(ses->host);
 	free(ses->port);
 	init_buffer(ses, 2);
 	free(ses->buffer);
 
-	if (ses == gtd->ses)
-	{
-		gtd->ses = newactive_session();
-	}
-
 	free(ses);
 
 	pop_call();
 	return;
 }
-
