@@ -78,29 +78,28 @@ DO_COMMAND(do_substitute)
 
 	if (*left == 0)
 	{
-		tintin_header(ses, " SUBSTITUTIONS ");
-		show_list(ses, root, PRIORITY);
+		show_list(ses, root, LIST_SUBSTITUTE);
 	}
 	else if (*left && !*right)
 	{
-		if (show_node_with_wild(ses, root, left, PRIORITY) == TRUE)
+		if (show_node_with_wild(ses, left, LIST_SUBSTITUTE) == TRUE)
 		{
 			;
 		}
-		else if (show_message(root))
+		else if (show_message(ses, LIST_SUBSTITUTE))
 		{
 			tintin_puts2("#THAT SUBSTITUTE IS NOT DEFINED.", ses);
 		}
 	}
 	else
 	{
-		updatenode_list(root, left, right, pr, PRIORITY);
+		updatenode_list(ses, left, right, pr, LIST_SUBSTITUTE);
 
-		if (show_message(root))
+		if (show_message(ses, LIST_SUBSTITUTE))
 		{
 			if (strcmp(right, "."))
 			{
-				tintin_printf2(ses, "#Ok. {%s} is substituted as {%s} @ {%s}.", right, left, pr);
+				tintin_printf2(ses, "#Ok. {%s} is substituted as {%s} @ {%s}.", left, right, pr);
 			}
 			else
 			{
@@ -125,7 +124,7 @@ DO_COMMAND(do_unsubstitute)
 
 	while ((node = search_node_with_wild(root, left)))
 	{
-		if (show_message(root))
+		if (show_message(ses, LIST_SUBSTITUTE))
 		{
 			if (strcmp(node->right, "."))
 			{
@@ -137,11 +136,11 @@ DO_COMMAND(do_unsubstitute)
 			}
 
 		}
-		deletenode_list(root, node);
+		deletenode_list(ses, node, LIST_SUBSTITUTE);
 
 		flag = TRUE;
 	}
-	if (!flag && show_message(root))
+	if (!flag && show_message(ses, LIST_SUBSTITUTE))
 	{
 		tintin_puts2("#THAT SUBSTITUTE IS NOT DEFINED.", ses);
 	}
@@ -156,7 +155,7 @@ void check_all_substitutions(char *original, char *line, struct session *ses)
 
 	for (node = ses->list[LIST_SUBSTITUTE]->f_node ; node ; node = node->next)
 	{
-		if (check_one_action(line, node->left, ses))
+		if (check_one_action(line, original, node->left, ses))
 		{
 			if (strcmp(node->right, "."))
 			{
