@@ -148,8 +148,9 @@ void translate_telopts(struct session *ses, unsigned char *src, int cplen)
 				break;
 
 			default:
-				tintin_puts2("\n\r#COMPRESSION ERROR, DISABLING MCCP.", ses);
+				tintin_puts2("\n\r#COMPRESSION ERROR, RESETTING MCCP.", ses);
 				send_dont_mccp2(ses);
+				send_do_mccp2(ses);
 				inflateEnd(ses->mccp);
 				free(ses->mccp);
 				ses->mccp = NULL;
@@ -287,13 +288,49 @@ void translate_telopts(struct session *ses, unsigned char *src, int cplen)
 			{
 				case '\0':
 				case '\r':
-					cpsrc++;
 					break;
+
 				default:
-					gtd->mud_output_len++;
-					*cpdst++ = *cpsrc++;
+					if (HAS_BIT(gtd->ses->flags, SES_FLAG_CONVERTMETA))
+					{
+						switch (*cpsrc)
+						{
+							case '\e':
+								*cpdst++ = '\\';
+								*cpdst++ = 'e';
+								gtd->mud_output_len += 2;
+								break;
+
+							case '\n':
+								*cpdst++ = *cpsrc;
+								gtd->mud_output_len++;
+								break;
+
+							default:
+								if (*cpsrc < 27)
+								{
+									*cpdst++ = '\\';
+									*cpdst++ = 'C';
+									*cpdst++ = '-';
+									*cpdst++ = 'a' + *cpsrc - 1;
+									gtd->mud_output_len += 4;
+								}
+								else
+								{
+									*cpdst++ = *cpsrc;
+									gtd->mud_output_len++;
+								}
+								break;
+						}
+					}
+					else
+					{
+						*cpdst++ = *cpsrc;
+						gtd->mud_output_len++;
+					}
 					break;
 			}
+			cpsrc++;
 			cplen--;
 		}
 	}
@@ -414,8 +451,6 @@ void send_echo_on(struct session *ses)
 
 	SET_BIT(ses->flags, SES_FLAG_LOCALECHO);
 
-	readline_echoing_p = TRUE;
-
 	telopt_debug(ses, "SENT IAC DONT ECHO");
 }
 
@@ -425,8 +460,6 @@ void send_echo_off(struct session *ses)
 
 	DEL_BIT(ses->flags, SES_FLAG_LOCALECHO);
 
-	readline_echoing_p = FALSE;
-
 	telopt_debug(ses, "SENT IAC DO ECHO");
 }
 
@@ -493,22 +526,6 @@ void zlib_free(void *opaque, void *address)
 	free(address);
 }
 
-
-DO_COMMAND(do_debugtelnet) 
-{
-	TOG_BIT(ses->flags,SES_FLAG_DEBUGTELNET);
-
-	if (HAS_BIT(ses->flags,SES_FLAG_DEBUGTELNET))
-	{
-  		tintin_puts2("#DEBUGTELNET IS NOW ON",ses);
-	}
-	else
-	{
-		tintin_puts2("#DEBUGTELNET IS NOW OFF",ses);
-	}
-	return ses;
-}
-
 void init_telnet_session(struct session *ses)
 {
 	send_do_sga(ses);
