@@ -33,7 +33,7 @@
 * Simple regexp that skips vt102 codes and supports the * and ? wildcards     *
 ******************************************************************************/
 
-bool regexp(const char *exp, const char *str)
+int regexp(const char *exp, const char *str)
 {
 	short cnt;
 
@@ -258,9 +258,24 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 				{
 					for (cnt = 0 ; gtd->vars[pti[1] - '0'][cnt] ; cnt++)
 					{
-						if (HAS_BIT(flags, SUB_SEC) && gtd->vars[pti[1] - '0'][cnt] == ';')
+						if (HAS_BIT(flags, SUB_SEC))
 						{
-							*pto++ = ':';
+							switch (gtd->vars[pti[1] - '0'][cnt])
+							{
+								case ';':
+									*pto++ = '\\';
+									*pto++ = ';';
+									break;
+
+								case '\\':
+									*pto++ = '\\';
+									*pto++ = '\\';
+									break;
+
+								default:
+									*pto++ = gtd->vars[pti[1] - '0'][cnt];
+									break;
+							}
 						}
 						else
 						{
@@ -286,6 +301,18 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 				}
 				break;
 
+			case '~':
+				if (HAS_BIT(flags, SUB_ANC) && pti == string)
+				{
+					string++;
+					pti++;
+				}
+				else
+				{
+					*pto++ = *pti++;
+				}
+				break;
+
 			case '&':
 				if (HAS_BIT(flags, SUB_CMD) && isdigit(pti[1]))
 				{
@@ -355,35 +382,49 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 * string is compared unless it starts or ends with the ^ character.           *
 ******************************************************************************/
 
-bool check_one_action(const char *line, const char *action, struct session *ses)
+int check_one_action(const char *line, const char *original, const char *action, struct session *ses)
 {
 	char result[BUFFER_SIZE], *ptr;
 	const char *ptl;
 
-	substitute(ses, action, result, SUB_VAR|SUB_FUN);
+	substitute(ses, action, result, SUB_VAR|SUB_FUN|SUB_ESC);
 
-	if (result[0] == '^')
+	ptr = result;
+
+	if (*ptr == '~')
 	{
-		return action_regexp(&result[1], line);
+		ptr++;
+		ptl = original;
+	}
+	else
+	{
+		ptl = line;
 	}
 
-	if (result[0] == '%' && isdigit(result[1]))
+	if (*ptr == '^')
 	{
-		return action_regexp(result, line);
+		ptr++;
+		return action_regexp(ptr, ptl);
 	}
 
-	for (ptl = line, ptr = result ; *ptl ; ptl++)
+	if (ptr[0] == '%' && isdigit(ptr[1]))
+	{
+		return action_regexp(ptr, ptl);
+	}
+
+	while (*ptl)
 	{
 		if (*ptr == *ptl && action_regexp(ptr, ptl))
 		{
 			return TRUE;
 		}
+		ptl++;
 	}
 	return FALSE;
 }
 
 
-bool action_regexp(const char *exp, const char *str)
+int action_regexp(const char *exp, const char *str)
 {
 	short cnt;
 
