@@ -80,11 +80,10 @@
 /*
 	Typedefs
 */
-typedef unsigned char bool;
 
 typedef struct session *COMMAND (struct session *ses, const char *arg);
-typedef struct session *CONFIG (struct session *ses, char *arg, int index);
-
+typedef struct session *CONFIG  (struct session *ses, char *arg, int index);
+typedef struct session *CLASS   (struct session *ses, char *arg);
 
 /*
 	A bunch of constants
@@ -98,15 +97,11 @@ typedef struct session *CONFIG (struct session *ses, char *arg, int index);
 
 #define	PRIORITY                0
 #define	ALPHA                   1
-#define   APPEND                  2
+#define APPEND                  2
 
 #define	DEFAULT_OPEN            '{'
 #define	DEFAULT_CLOSE           '}'
 
-#ifndef	DEFAULT_FILE_DIR
-#define	DEFAULT_FILE_DIR        "/usr/local/lib/tintin"
-#endif
-
 #define HISTORY_FILE              ".tt_history"
 
 #define DEFAULT_CAT_STR           "cat "
@@ -121,7 +116,6 @@ typedef struct session *CONFIG (struct session *ses, char *arg, int index);
 #define FILE_SIZE                150000
 #define BUFFER_SIZE               10000
 #define NUMBER_SIZE                 100
-#define BUFFER_SAFE                   4
 
 /*
 	NOTE: get rid of the DEVELOPMENT warning in main() when you update this!
@@ -165,15 +159,19 @@ typedef struct session *CONFIG (struct session *ses, char *arg, int index);
 #define LIST_MACRO                    11
 #define LIST_PROMPT                   12
 #define LIST_TAB                      13
-#define LIST_MAX                      14
-#define LIST_MATH                     14
-#define LIST_DELAY                    15
-#define LIST_ALL                      16
+#define LIST_CLASS                    14
+#define LIST_MAX                      15
 
-#define CONFIG_MAX                    15
+#define LIST_MATH                     LIST_MAX + 0
+#define LIST_DELAY                    LIST_MAX + 1
+#define LIST_ALL                      LIST_MAX + 2
+
+#define CONFIG_MAX                    17
 
 #define COLOR_MAX                     24
 
+#define CLASS_MAX                      5
+
 /*
 	Various flags
 */
@@ -188,8 +186,6 @@ typedef struct session *CONFIG (struct session *ses, char *arg, int index);
 #define SUB_ANC                       (1 <<  6)
 #define SUB_SEC                       (1 <<  7)
 
-#define SUB_DEF                       (SUB_VAR|SUB_FUN)
-
 #define SES_FLAG_NAWS                 (1 <<  0)
 #define SES_FLAG_ECHOCOMMAND          (1 <<  1)
 #define SES_FLAG_SNOOP                (1 <<  2)
@@ -212,16 +208,27 @@ typedef struct session *CONFIG (struct session *ses, char *arg, int index);
 #define SES_FLAG_DEBUGTELNET          (1 << 19)
 #define SES_FLAG_USERCOMMAND          (1 << 20)
 #define SES_FLAG_SCROLLLOCK           (1 << 21)
-#define SES_FLAG_MORE_OUTPUT          (1 << 22)
+#define SES_FLAG_SCAN                 (1 << 22)
+#define SES_FLAG_SCROLLSTOP           (1 << 23)
+#define SES_FLAG_CONVERTMETA          (1 << 24)
+#define SES_FLAG_GAGPROMPT            (1 << 25)
+#define SES_FLAG_BREAK                (1 << 26)
 
 #define LIST_FLAG_IGNORE              (1 <<  0)
 #define LIST_FLAG_MESSAGE             (1 <<  1)
 #define LIST_FLAG_DEBUG               (1 <<  2)
-
 #define LIST_FLAG_DEFAULT             LIST_FLAG_MESSAGE
 
+#define NODE_FLAG_CLASS               (1 <<  0)
+#define NODE_FLAG_MAX                 (1 <<  1)
 
-#define MAX_COMMAND                   81
+#define STR_HASH_FLAG_NOGREP          (1 <<  0)
+
+#define CMD_FLAG_NONE                 (0 <<  0)
+#define CMD_FLAG_SUB                  (1 <<  0)
+
+
+#define MAX_COMMAND                   82
 
 #define MAX_STR_HASH                  5000
 
@@ -306,12 +313,13 @@ typedef struct session *CONFIG (struct session *ses, char *arg, int index);
 #define UMAX(a, b)                ((a) > (b) ? (a) : (b))
 #define UMIN(a, b)                ((a) < (b) ? (a) : (b))
 
-#define IS_SPLIT(ses)             ((ses)->rows != (ses)->bot_row)
+#define IS_SPLIT(ses)             ((ses)->rows != (ses)->bot_row || (ses)->top_row != 1)
 
 #define SCROLL(ses)               (((ses)->cur_row >= (ses)->top_row && (ses)->cur_row <= (ses)->bot_row) || (ses)->cur_row == (ses)->rows)
 
 #define DO_COMMAND(command) struct session  *command (struct session *ses, const char *arg)
 #define DO_CONFIG(config) struct session *config (struct session *ses, char *arg, int index)
+#define DO_CLASS(class) struct session *class (struct session *ses, char *arg)
 
 /*
 	Bit operations
@@ -322,6 +330,11 @@ typedef struct session *CONFIG (struct session *ses, char *arg, int index);
 #define DEL_BIT(bitvector, bit)   ((bitvector) &= (~(bit)))
 #define TOG_BIT(bitvector, bit)   ((bitvector) ^= (bit))
 
+/*
+	Compatibility
+*/
+
+#define atoll(str) (strtoll(str, NULL, 10))
 
 
 /************************ structures *********************/
@@ -339,6 +352,7 @@ struct listnode
 {
 	struct listnode   * next;
 	struct listnode   * prev;
+	struct listnode   * class;
 	char              * left;
 	char              * right;
 	char              * pr;
@@ -357,6 +371,7 @@ struct session
 	FILE                  * logfile;
 	FILE                  * logline;
 	struct listroot       * list[LIST_ALL];
+	struct listnode       * class;
 	int                     rows;
 	int                     cols;
 	int                     top_row;
@@ -372,7 +387,7 @@ struct session
 	int                     bgc;
 	int                     vtc;
 	int                     socket;
-	int                     flags;
+	long long               flags;
 	char                  * host;
 	char                  * port;
 	long long               connect_retry;
@@ -405,6 +420,7 @@ struct command_type
 {
 	char                  * name;
 	COMMAND               * command;
+	int                     flags;
 };
 
 struct list_type
@@ -431,12 +447,19 @@ struct color_type
 	char                  * code;
 };
 
+struct class_type
+{
+	char                  * name;
+	CLASS                 * class;
+};
+
 
 struct str_hash_data
 {
 	struct str_hash_data    * next;
 	struct str_hash_data    * prev;
-	int                       count;
+	unsigned short            count;
+        unsigned short            flags;
 	unsigned short            lines;
 	unsigned short            hash;
 };
@@ -470,6 +493,7 @@ struct exit_data
 
 #endif
 
+
 /*
 	Some stuff we use globally
 */
@@ -480,6 +504,28 @@ extern int readline_echoing_p;
 	Function declarations
 */
 
+#ifndef __ECHO_H__
+#define __ECHO_H__
+
+extern void echo_on(struct session *ses);
+extern void echo_off(struct session *ses);
+
+#endif
+
+#ifndef __CLASS_H__
+#define __CLASS_H__
+
+extern DO_COMMAND(do_class);
+extern DO_COMMAND(do_unclass);
+extern DO_CLASS(class_open);
+extern DO_CLASS(class_close);
+extern DO_CLASS(class_read);
+extern DO_CLASS(class_write);
+extern DO_CLASS(class_kill);
+extern void parse_class(struct session *ses, char *input, struct listnode *class);
+
+#endif
+
 #ifndef __MAPPER_H__
 #define __MAPPER_H__
 
@@ -507,7 +553,7 @@ extern void search_coord(int vnum, short x, short y, short z);
 extern DO_COMMAND(do_math);
 extern DO_COMMAND(do_if);
 extern long long mathexp(struct session *ses, const char *str);
-extern bool mathexp_tokenize(struct session *ses, const char *str);
+extern int mathexp_tokenize(struct session *ses, const char *str);
 extern void mathexp_level(struct listnode *node);
 extern void mathexp_compute(struct listnode *node);
 extern long long tintoi(const char *str);
@@ -535,10 +581,10 @@ extern void do_one_prompt(struct session *ses, char *prompt, int row);
 #ifndef __REGEXP_H__
 #define __REGEXP_H__
 
-extern bool regexp(const char *exp, const char *str);
+extern int regexp(const char *exp, const char *str);
 extern void substitute(struct session *ses, const char *string, char *result, int flags);
-extern bool check_one_action(const char *line, const char *action, struct session *ses);
-extern bool action_regexp(const char *exp, const char *str);
+extern int check_one_action(const char *line, const char *original, const char *action, struct session *ses);
+extern int action_regexp(const char *exp, const char *str);
 
 
 #endif
@@ -563,6 +609,8 @@ extern DO_CONFIG(config_packetpatch);
 extern DO_CONFIG(config_tintinchar);
 extern DO_CONFIG(config_verbatimchar);
 extern DO_CONFIG(config_repeatchar);
+extern DO_CONFIG(config_debugtelnet);
+extern DO_CONFIG(config_convertmeta);
 
 #endif
 
@@ -575,13 +623,15 @@ extern DO_COMMAND(do_unmacro);
 extern void macro_update(void);
 #endif
 
-#ifndef __STRALLOC_H__
-#define __STRALLOC_H__
+#ifndef __STRHASH_H__
+#define __STRHASH_H__
 
 extern char *str_hash(char *str, int lines);
 extern char *str_unhash(char *str);
-extern short str_hash_lines(char *str);
+extern unsigned short str_hash_lines(char *str);
+extern unsigned short str_hash_grep(char *str, int write);
 extern void reset_hash_table(void);
+
 DO_COMMAND(do_hash);
 
 #endif
@@ -590,7 +640,7 @@ DO_COMMAND(do_hash);
 #define __BUFFER_H__
 
 extern void init_buffer(struct session *ses, int size);
-extern void add_line_buffer(struct session *ses, const char *line, bool more_output);
+extern void add_line_buffer(struct session *ses, const char *line, int more_output);
 extern DO_COMMAND(do_buffer);
 extern DO_COMMAND(do_grep);
 extern int show_buffer(struct session *ses);
@@ -615,8 +665,7 @@ extern void dump_stack(void);
 extern struct session *do_action(struct session *ses, const char *arg);
 extern struct session *do_unaction(struct session *ses, const char *arg);
 
-extern bool check_presub_actions(const char *original, char *line, struct session *ses);
-extern bool check_postsub_actions(const char *original, char *line, struct session *ses);
+extern void check_all_actions(const char *original, char *line, struct session *ses);
 
 #endif
 
@@ -633,7 +682,7 @@ extern struct session *do_unalias(struct session *ses, const char *arg);
 
 extern DO_COMMAND(do_antisubstitute);
 extern DO_COMMAND(do_unantisubstitute);
-extern bool check_all_antisubstitutions(const char *original, char *line, struct session *ses);
+extern int check_all_antisubstitutions(const char *original, char *line, struct session *ses);
 
 #endif
 
@@ -642,14 +691,15 @@ extern bool check_all_antisubstitutions(const char *original, char *line, struct
 #define __FILES_H__
 
 extern DO_COMMAND(do_read);
+extern struct session *readfile(struct session *ses, const char *arg, struct listnode *class);
 extern DO_COMMAND(do_readmap);
-extern DO_COMMAND(do_readnew);
 extern DO_COMMAND(do_writesession);
 extern DO_COMMAND(do_write);
 extern DO_COMMAND(do_writemap);
 extern DO_COMMAND(do_session);
 extern void prepare_for_write(int mode, struct listnode *node, char *result);
 DO_COMMAND(do_textin);
+DO_COMMAND(do_scan);
 
 #endif 
 
@@ -678,7 +728,7 @@ extern DO_COMMAND(do_highlight);
 extern int is_high_arg(const char *s);
 extern DO_COMMAND(do_unhighlight);
 extern void check_all_highlights(char *original, char *line, struct session *ses);
-extern bool get_highlight_codes(const char *htype, char *result);
+extern int get_highlight_codes(const char *htype, char *result);
 
 #endif
 
@@ -687,17 +737,17 @@ extern bool get_highlight_codes(const char *htype, char *result);
 #define __LLIST_H__
 
 extern struct listroot *init_list(void);
-extern void kill_list(struct listroot *nptr);
+extern void kill_list(struct session *ses, int index);
 extern DO_COMMAND(do_killall);
-extern struct listroot *copy_list(struct listroot *sourcelist, int mode);
-extern void insertnode_list(struct listroot *listhead, const char *ltext, const char *rtext, const char *prtext, int mode);
-extern void updatenode_list(struct listroot *listhead, const char *ltext, const char *rtext, const char *prtext, int mode);
-extern void deletenode_list(struct listroot *listhead, struct listnode *nptr);
+extern struct listroot *copy_list(struct session *ses, struct listroot *sourcelist, int index);
+extern void insertnode_list(struct session *ses, const char *ltext, const char *rtext, const char *prtext, int ondex);
+extern void updatenode_list(struct session *ses, const char *ltext, const char *rtext, const char *prtext, int index);
+extern void deletenode_list(struct session *ses, struct listnode *node, int index);
 extern struct listnode *searchnode_list(struct listroot *listhead, const char *cptr);
 extern struct listnode *searchnode_list_begin(struct listroot *listhead, const char *cptr, int mode);
-extern void shownode_list(struct session *ses, struct listnode *nptr, int mode);
-extern void show_list(struct session *ses, struct listroot *listhead, int mode);
-extern int show_node_with_wild(struct session *ses, struct listroot *listhead, const char *cptr, int mode);
+extern void shownode_list(struct session *ses, struct listnode *node, int index);
+extern void show_list(struct session *ses, struct listroot *listhead, int index);
+extern int show_node_with_wild(struct session *ses, const char *cptr, int index);
 extern struct listnode *search_node_with_wild(struct listroot *listhead, const char *cptr);
 extern void addnode_list(struct listroot *listhead, const char *ltext, const char *rtext, const char *prtext);
 extern int count_list(struct listroot *listhead);
@@ -748,6 +798,7 @@ extern DO_COMMAND(do_forall);
 extern DO_COMMAND(do_message);
 extern DO_COMMAND(do_ignore);
 extern DO_COMMAND(do_debug);
+extern DO_COMMAND(do_return);
 extern DO_COMMAND(do_snoop);
 extern DO_COMMAND(do_system);
 extern DO_COMMAND(do_zap);
@@ -790,7 +841,6 @@ extern void send_dont_mccp2(struct session *ses);
 extern void init_mccp(struct session *ses);
 extern void *zlib_alloc(void *opaque, unsigned int items, unsigned int size);
 extern void zlib_free(void *opaque, void *address);
-extern DO_COMMAND(do_debugtelnet);
 
 #endif
 
@@ -837,7 +887,9 @@ extern void mainloop(void);
 extern void readmud(struct session *ses);
 extern void bait(void);
 extern void process_mud_output(struct session *ses, char *linebuf, int prompt);
-extern int  show_message(struct listroot *root);
+extern char *readkeyboard(void);
+extern void echo_command(struct session *ses, char *line, int force);
+extern int  show_message(struct session *ses, int index);
 extern void tintin_header(struct session *ses, const char *format, ...);
 extern void socket_printf(struct session *ses, size_t length, const char *format, ...);
 extern void tintin_printf2(struct session *ses, const char *format, ...);
@@ -905,10 +957,12 @@ extern void init_telnet_session(struct session *ses);
 extern struct session *session_command(const char *arg, struct session *ses);
 extern void show_session(struct session *ses, struct session *ptr);
 extern struct session *newactive_session(void);
+extern struct session *activate_session(struct session *ses);
 extern struct session *new_session(const char *name, const char *address, struct session *ses);
 extern void connect_session(struct session *ses);
 extern void cleanup_session(struct session *ses);
 
+
 #endif
 
 #ifndef __SUBSTITUTE_H__
@@ -930,6 +984,7 @@ extern const struct command_type command_table[MAX_COMMAND];
 extern const struct list_type list_table[LIST_ALL];
 extern const struct config_type config_table[CONFIG_MAX];
 extern const struct color_type color_table[COLOR_MAX];
+extern const struct class_type class_table[CLASS_MAX];
 
 #endif
 
