@@ -72,7 +72,15 @@ long long utime()
 
 	gettimeofday(&now_time, NULL);
 
-	return now_time.tv_sec * 1000000LL + now_time.tv_usec;
+	if (gtd->time >= now_time.tv_sec * 1000000LL + now_time.tv_usec)
+	{
+		gtd->time++;
+	}
+	else
+	{
+		gtd->time = now_time.tv_sec * 1000000LL + now_time.tv_usec;
+	}
+	return gtd->time;
 }
 
 
@@ -134,7 +142,7 @@ int strncasecmp(const char *string1, const char *string2, size_t count)
 				return(UPPER(*string1) - UPPER(*string2));
 					if (!*string1++)
 						break;
-		string2++:
+		string2++;
 	}
 	while (--count);
 
